package alarmProxy.impl;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Enumeration;
import java.util.Properties;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import alarmProxy.ServiceInterface;


public class Service implements ServiceInterface {

	private String copeurl = "http://localhost:8080/axis/services/copeserver";

	private String serviceurl = "http://localhost:8080/axis/services/alarmservice";

	private String name = "Alarm Service";

	protected File icon;

	private long id;

	private JPanel holder;

	protected JFrame main;

	private String copeIP;

	public Service() {
		System.out.println(serviceurl);
	}

	public void event(String description, Object oldValue, Object newValue) {
		System.out.println(getName() + " received command: event");
	}

	public void openInterface() {
		System.out.println(getName() + " received command: openInterface");
		main = new JFrame("Alarm Service");

		main.addWindowListener(new WindowAdapter() {

			public void windowClosing(WindowEvent e) {
				System.out.println("windowClosing");
				unsubscribe();
			}

			public void windowOpened(WindowEvent e) {
				System.out.println("windowOpened");
				subscribe();
			}
		});

		holder = new JPanel();
		main.setSize(400, 300);
		main.setContentPane(holder);
		main.setVisible(true);
	}

	public Properties getDescription() {
		System.out.println(getName() + " received command: getDescription");
		Properties props = new Properties();
		props.put("connection.terminating.1", "alarmConnection");
		props.put("connection.originating.1", "smsConnection");
		props.put("connection.originating.2", "ttsConnection");
		
		return props;
	}

	public void connect(String description, String source, String target) {
		System.out.println(getName() + " received command: connect");
		Communicator.sendRequest(serviceurl, "connect", new Object[] { description, source, target });	
	}

	public void disconnect(String description, String source, String target) {
		System.out.println(getName() + " received command: disconnect");
		Communicator.sendRequest(serviceurl, "disconnect", new Object[] { description, source, target });
	}

	public void setID(long id) {
		System.out.println(getName() + " received command: setID");
		this.id = id;
	}

	public void ping() {
		System.out.print(getName() + " received ping");
		Communicator.sendRequest(copeurl, "pong", new Object[] { new Long(id) });
	}

	public void reRegisterWithCope() {
		System.out.println(getName() + " received command: reRegisterWithCope");
		Communicator.sendRequest(copeurl, "reRegisterService", new Object[] { new Long(id) });
	}

	public String getName() {
		return name;
	}

	public void setURL(String url) {
		serviceurl = url;
	}

	public String getURL() {
		return serviceurl;
	}

	public String getIconPath() {
		try {
			if (icon.exists())
				icon.delete();

			icon.createNewFile();

			FileInputStream fis = (FileInputStream) getClass()
					.getResourceAsStream("/icon.png");
			FileOutputStream fos = new FileOutputStream(icon);
			int b;
			while ((b = fis.read()) != -1) {
				fos.write(b);
			}

			fos.flush();
			fos.close();

			fis.close();

			if (icon != null)
				return icon.getPath();

		} catch (IOException e) {
			e.printStackTrace();
		}

		return null;
	}

	
	/************************************
	 * 
	 * Custom stuff
	 *
	 ************************************/
	public void subscribe() {
/*
 * This does NOT ensure use of correct IP address when multiple network interfaces are installed.
 */
//		try {
//			InetAddress i = InetAddress.getLocalHost();
//			copeIP = i.getHostAddress();
//			System.out.println("copeIP sat by InetAddress");
//			
//		} catch (UnknownHostException e1) {
//			System.out.println(e1.getMessage());
//		}
		// getting ip from cope server
		copeIP = (String) Communicator.sendRequest(copeurl, "getIp", new Object[]{});
		// subscribing with correct IP
//		Communicator.sendRequest(serviceurl, "subscribe", new Object[] { "http://" + copeIP + ":8080/axis/services/alarmproxy" });
		Communicator.sendRequest(serviceurl, "subscribe", new Object[] { "http://localhost:8080/axis/services/alarmproxy" });
	}

	private void unsubscribe() {
//		Communicator.sendRequest(serviceurl, "unsubscribe", new Object[] { "http://" + copeIP + ":8080/axis/services/alarmproxy" });
		Communicator.sendRequest(serviceurl, "unsubscribe", new Object[] { "http://localhost:8080/axis/services/alarmproxy" });
	}

	public void update(String[][] inputs, String[] outputs) {
		System.out.println(getName() + " received update (" + inputs.length + " items)");
		holder.removeAll();
		holder.setLayout(new GridLayout(0, 1));
		
		for (int i = 0; i < inputs.length; i++) {
			JPanel layerPanel = new JPanel();
			layerPanel.setLayout(new GridLayout(0, 4));
			
			layerPanel.add(new JLabel(inputs[i][0]));
			
			JComboBox cb = new JComboBox(outputs);	
			cb.setSelectedItem(inputs[i][1]);
			layerPanel.add(cb);
			
			layerPanel.add(new JTextField(inputs[i][2]));
			
			JButton button = new JButton("Update");
			button.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					JButton aButton = (JButton) e.getSource();
					Container parent = aButton.getParent();
					Component[] comps = parent.getComponents();
					JLabel source = (JLabel) comps[0];
					JComboBox combo = (JComboBox) comps[1];
					JTextField address = (JTextField) comps[2];
					System.out.println("Components in selected panel: " + comps);
					updateService(source.getText(), (String) combo.getSelectedItem(), address.getText());
				}
			});
			
			layerPanel.add(button);

			holder.add(layerPanel);
		}
		
		main.pack();
		main.repaint();

		holder.updateUI();
	}

	protected void updateService(String source, String outputConnection, String address) {
		System.out.println("Updating " + getName() + " with " + source + " --" + outputConnection + "--> " + address);
//		updateService(String source, String outputConnection, String address)
		Communicator.sendRequest(serviceurl, "updateService", new Object[]{source, outputConnection, address});
	}

}
