package alarmService.impl;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;

import alarmService.ServiceInterface;

public class Service implements ServiceInterface {

	List inputConnections = new ArrayList();

	Hashtable outputConnections = new Hashtable();

	Hashtable router = new Hashtable();
	
	List subscribers = new ArrayList();

	public void connect(String description, String source, String target) {
		// if description == alarmconnection we know this is the target 
		// someService --alarmConnection--> alarmService
		if(description.equalsIgnoreCase("alarmconnection")) {
			if (!inputConnections.contains(source)) {
				inputConnections.add(source);
				System.out.println("Alarm Service: Added input connection: " + source);
				updateSubscribers();
			}
		} else {
			if (!outputConnections.containsKey(description)) {
				outputConnections.put(description, target);
				System.out.println("Alarm Service: Added output connection: " + description + " -> " + outputConnections.get(description));
				updateSubscribers();
			}
		}
	}

	public void disconnect(String description, String source, String target) {
		// if description == alarmconnection we know this is the target 
		// someService --alarmConnection--> alarmService
		if(description.equalsIgnoreCase("alarmconnection")) {
			if (inputConnections.contains(source)) {
				inputConnections.remove(source);
				System.out.println("Alarm Service: Removed input connection: " + source);
				updateSubscribers();
			}
		} else {
			if (outputConnections.containsKey(description)) {
				outputConnections.remove(description);
				System.out.println("Alarm Service: Removed output connection: " + description);
				updateSubscribers();
			}
		}
	}

	public void sendAlarm(String alarm, String source) {
		if(router.containsKey(source)) {
			String[] output = (String[]) router.get(source);
			if(output[0].equalsIgnoreCase("smsConnection")) {
				System.out.println("Sending SMS alarm with " + alarm + " to " + output[1]);
				Communicator.sendRequest((String) outputConnections.get(output[0]), "sendSms", new Object[]{ alarm, output[1]});
				return;
			}
			if(output[0].equalsIgnoreCase("ttsConnection")) {
				System.out.println("Sending TTS alarm " + alarm + " to " + output[1]);
				Communicator.sendRequest((String) outputConnections.get(output[0]), "speak", new Object[]{ alarm });
				return;
			}
		}
	}

	public void subscribe(String address) {
		if (!subscribers.contains(address))
			subscribers.add(address);
		updateSubscriber(address);
	}
	
	public void unsubscribe(String address) {
		if (subscribers.contains(address))
			subscribers.remove(address);
	}

	/*
	 * For the proxy to set correct output for an alarm
	 */
	public void updateService(String source, String outputConnection, String address) {
		router.put(source, new String[]{outputConnection, address});
		updateSubscribers();
	}

	/** Method for updating only one subscriber
	 * @param address
	 */
	private void updateSubscriber(String address) {
		String[][] inputs = new String[inputConnections.size()][3];
		
		for (int i = 0; i < inputConnections.size(); i++) {
			inputs[i][0] = (String) inputConnections.get(i);
			if(router.containsKey(inputs[i][0])) {
				String[] route = (String[]) router.get(inputs[i][0]);
				inputs[i][1] = route[0];
				inputs[i][2] = route[1];
			} else {
				inputs[i][1] = "";
				inputs[i][2] = "";
			}
		}
		
		String[] outputs = new String[outputConnections.size()];
		int index = 0;
		Enumeration keys = outputConnections.keys();
		while (keys.hasMoreElements()) {
			String key = (String) keys.nextElement();
			outputs[index++] = key;
		}
		
		Communicator.sendRequest(address, "update", new Object[]{inputs, outputs});
	}

	/** Method for updating all subscribers
	 * 
	 */
	private void updateSubscribers() {
		String[][] inputs = new String[inputConnections.size()][3];
		
		for (int i = 0; i < inputConnections.size(); i++) {
			inputs[i][0] = (String) inputConnections.get(i);
			if(router.containsKey(inputs[i][0])) {
				String[] route = (String[]) router.get(inputs[i][0]);
				inputs[i][1] = route[0];
				inputs[i][2] = route[1];
			} else {
				inputs[i][1] = "";
				inputs[i][2] = "";
			}
		}
		
		String[] outputs = new String[outputConnections.size()];
		int index = 0;
		Enumeration keys = outputConnections.keys();
		while (keys.hasMoreElements()) {
			String key = (String) keys.nextElement();
			outputs[index++] = key;
		}
		
		for (int i = 0; i < subscribers.size(); i++) {
			Communicator.sendRequest((String)subscribers.get(i), "update", new Object[]{inputs, outputs});
		}
	}
}
