package copeServer.impl;

import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleEvent;
import org.osgi.framework.BundleListener;

import copeServer.COPEi;
import copeServer.impl.composition.Composition;
import copeServer.impl.composition.CompositionRegistry;
import copeServer.impl.device.Device;
import copeServer.impl.discovery.DiscoveryManager;
import copeServer.impl.gui.WindowManager;
import copeServer.impl.ontology.OntologyManager;
import copeServer.impl.service.Service;
import copeServer.impl.service.ServiceHandler;
import copeServer.impl.service.ServiceRegistry;

public class COPE implements COPEi{
	
	private BundleContext bundleContext;
	
	private Device device;
	private ServiceRegistry serviceRegistry;
	private ServiceHandler serviceHandler;
	private DiscoveryManager discoveryManager;
	private OntologyManager ontologyManager;
	private CompositionRegistry compositionRegistry;
	private WindowManager windowManager;
	
	public COPE(){
	
	}
	
	public void init(BundleContext context){
		this.bundleContext = context;
		System.out.println("Loading device information...");
		device = new Device();
		System.out.println("Loading service registry...");
		serviceRegistry = new ServiceRegistry(context);
		System.out.println("Loading service discovery...");
		discoveryManager = new DiscoveryManager();
		System.out.println("Loading service ontology...");
		ontologyManager = new OntologyManager(serviceRegistry);
		System.out.println("Loading composition registry...");
		compositionRegistry = new CompositionRegistry();
		System.out.println("Loading service handler...");
		serviceHandler = new ServiceHandler(serviceRegistry, compositionRegistry, ontologyManager, context);
		System.out.println("Loading bundle listener...");
		bundleContext.addBundleListener(new BundleListener(){
			public void bundleChanged(BundleEvent arg0) {
				long bid = arg0.getBundle().getBundleId();
				if(serviceRegistry.getService(bid)!=null){
					serviceRegistry.getService(bid).setState(arg0.getBundle().getState());
					if(arg0.getType()==BundleEvent.UNINSTALLED){
						System.out.println("Removing");
						serviceRegistry.removeService(serviceRegistry.getService(bid));
						System.out.println("Finished removing");
					}
				}
			}
		});
		System.out.println("Loading user interface...");
		windowManager = new WindowManager(device, serviceRegistry, serviceHandler, discoveryManager, ontologyManager, compositionRegistry);
		System.out.println("System ready");
	}
	
	public void exit(boolean savesession){
		if(savesession){
			System.out.println("Saving session...");
			serviceHandler.saveAll();
		}
		System.out.println("Disconnecting compositions...");
		for (int i = 0; i < compositionRegistry.getSize(); i++) {
			((Composition)compositionRegistry.getElementAt(i)).disconnectAll();
		}
		System.out.println("Exiting cope...");
		windowManager.dispose();
	}
	
	/*
	 * Following are the published SOAP methods...
	 */
	
	//public method for ping reply from services
	public void pong(long source){
		serviceRegistry.pong(source);
	}
	
	/*
	 *  Methods for testing server
	 */
	public void printServer(){
		System.out.println("---Status---");
		System.out.println("Printing Services");
		for (int i = 0; i < serviceRegistry.getSize(); i++) {
			Service service = (Service)serviceRegistry.getElementAt(i);
			System.out.println(service);
		}
		System.out.println("Done");
	}
	
	public void printClient(){
		System.out.println("---Client---");
		for (int i = 0; i < serviceRegistry.getSize(); i++) {
			Service service = (Service)serviceRegistry.getElementAt(i);
			System.out.println("Pinging "+service);
			service.ping();
		}
	}

	public void event(int source, String description, Object oldValue, Object newValue) {
		// TODO Auto-generated method stub
		
	}

	public void registerService(String url, int connection) {
		// TODO Auto-generated method stub
		
	}

	public String getIp() {
		return device.getIp();
	}

	public void reRegisterService(long bid) {
		serviceHandler.reRegisterService(bid);
	}
}
