package copeServer.impl.composition;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.List;

import org.osgi.framework.BundleEvent;

import copeServer.impl.ontology.OntologyManager;
import copeServer.impl.service.Service;

public class ComposedService implements PropertyChangeListener{

	private Service origin;
	private List relations = new ArrayList();
	private String x;
	private String y;
	
	private PropertyChangeSupport pcs;
	
	public ComposedService(Service service){
		this.origin = service;
		origin.addPropertyChangeListener(this);
	}
	
	public Service getService(){
		return origin;
	}

//	/**
//	 * Calling this method initiates the current setup of the relation.
//	 * Fires ("connect", null, null).
//	 */
//	public void connect(){
//		String[] desc = new String[connections.size()];
//		String[] dest = new String[connections.size()];
//		
//		for (int i = 0; i < connections.size(); i++) {
//				desc[i] = ((Relation)connections.get(i)).description;
//				dest[i] = ((Relation)connections.get(i)).target;
//		}
//		System.out.println("RELATION: origin.connect("+desc.length+", "+dest.length+"); first value="+((Relation)connections.get(0)).target);
//		origin.connect(desc, dest);
//		
//		origin.connect(connection.description, getURLs());
//		firePropertyChangeEvent("connect", null, null);
//	}
//	
//	/**
//	 * Disconnect the given url on this Relation's origin Service
//	 * @param url
//	 */
//	public void disconnect(String url){
//		System.out.println("RELATION: origin.disconnect("+url);
//		origin.disconnect(url);
//		firePropertyChangeEvent("connect", null, null);
//	}
//
//	/**
//	 * @return String[] of all currently set connections.
//	 */
//	public String[] getURLs() {
//		String[] ret = new String[connections.size()];
//		for (int i = 0; i < ret.length; i++){
//			ret[i] = ((Relation) connections.get(i)).target;
//		}
//		return ret;
//	}
//	
//	/**
//	 * Add a destination connection to this relation.
//	 * Fires ("destinationchange", old connection, new connection).
//	 * @param description - Description of the connection (needs to be handled by the service).
//	 * @param destination - The endpoint to the selected relation.
//	 * @param removeExisting - Defines if the method should remove all existing connections before adding "destination".
//	 */
//	public void setConnection(String description, String destination, boolean removeExisting){
//		
//		if(removeExisting || connections==null){
//			connections = new ArrayList();
//		}
//		if(description != null || destination != null){
//			Relation connection = new Relation(description, destination);
//			connections.add(connection);
//		}
////		firePropertyChangeEvent("destinationchange", oldValue, connections);
//	}
	
	public Relation getRelation(int number){
		return (Relation) relations.get(number);
	}
	
	public List getRelations(){
		return relations;
	}
	
	public void addRelation(Relation relation){
		if(!this.relations.contains(relation)) {
			this.relations.add(relation);
			relation.addPropertyChangeListener(this);
		}
	}
	
	public void removeRelation(Relation relation){
		if(this.relations.contains(relation)) {
			this.relations.remove(relation);
			relation.removePropertyChangeListener(this);
		}
	}

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		if (pcs == null) {
			pcs = new PropertyChangeSupport(this);
		}
		pcs.addPropertyChangeListener(listener);
	}
	public void removePropertyChangeListener(PropertyChangeListener listener) {
		if (pcs != null) {
			pcs.removePropertyChangeListener(listener);
		}
	}
	
	protected void firePropertyChangeEvent(String propertyName, Object oldValue, Object newValue) {
		if (pcs == null) {
			return;
		}
		pcs.firePropertyChange(propertyName, oldValue, newValue);
	}
	
	public String toString(){
		return origin.toString();
	}

	public String getX() {
		return x;
	}

	public void setX(String x) {
		String oldX = this.x;
		this.x = x;
		firePropertyChangeEvent("x", oldX, x);
	}

	public String getY() {
		return y;
	}

	public void setY(String y) {
		String oldY = this.y;
		this.y = y;
		firePropertyChangeEvent("y", oldY, y);
	}

	public void propertyChange(PropertyChangeEvent arg0) {
		if(arg0.getPropertyName().equalsIgnoreCase("uninstalled")){
			//a service has been uninstalled
		}
		if(arg0.getPropertyName().equalsIgnoreCase("state")){
			firePropertyChangeEvent("state", arg0.getOldValue(), arg0.getNewValue());
		}
		if(arg0.getPropertyName().equalsIgnoreCase("relationactivation")){
			//A relation has been activated or deactivated
		}
		
	}

	public void disconnectAll() {
		for (int i = 0; i < relations.size(); i++) {
			Relation relation = (Relation) relations.get(i);
			relation.setActive(false);
		}
	}

	public void disconnect(Relation relation) {
		relation.setActive(false);
	}

	public void connectAll() {
		for (int i = 0; i < relations.size(); i++) {
			Relation relation = (Relation) relations.get(i);
			relation.setActive(true);
		}
	}
	
	public void connect(Relation relation) {
		relation.setActive(true);
	}
	
}