package copeServer.impl.composition;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractListModel;

import copeServer.impl.service.Service;

public class Composition extends AbstractListModel implements PropertyChangeListener{

	private List composedServices = new ArrayList();
	
	private PropertyChangeSupport pcs;
	
	/**
	 * Add a Relation
	 * @param relation
	 */
	public void addRelation(ComposedService relation){
		if(!composedServices.contains(relation)){
			composedServices.add(relation);
			relation.addPropertyChangeListener(this);
			fireIntervalAdded(this, 0, getSize());
			System.out.println("Added new relation: "+relation);
		} else{
			System.out.println("ERROR ! Trying to add relation that is allready there.");
		}
	}
	
	/**
	 * Remove a Relation
	 * @param relation
	 */
	public void removeRelation(ComposedService relation){
		if(composedServices.contains(relation)){
			composedServices.remove(relation);
			relation.removePropertyChangeListener(this);
			fireIntervalRemoved(this, 0, getSize());
			System.out.println("Removed relation "+relation);
		} else{
			System.out.println("ERROR ! Trying to remove relation that is not there.");
		}
	}
	
	public int getSize() {
		return composedServices.size();
	}

	public Object getElementAt(int arg0) {
		return composedServices.get(arg0);
	}
	
	public void propertyChange(PropertyChangeEvent evt) {
		ComposedService service = (ComposedService)evt.getSource();
		if("connect".equals(evt.getPropertyName()) || "destinationchange".equals(evt.getPropertyName())){
			int pos = composedServices.indexOf(service);
			if (pos >= 0) {
				System.out.println("Fireing contentsChanged in position: "+pos);
				fireContentsChanged(this, pos, pos);
			} else{
				System.out.println("Error ! Wrong in position.");
			}
		}
	}

	/**
	 * Retrieve a ComposedService
	 * @param position
	 * @return Relation of a composition.
	 */
	public ComposedService getComposedService(int position){
		return (ComposedService) composedServices.get(position);
	}
	
	public void disconnectAll(){
		for (int i = 0; i < composedServices.size(); i++) {
			getComposedService(i).disconnectAll();
		}
	}
	
	public void connectAll(){
		for (int i = 0; i < composedServices.size(); i++) {
			getComposedService(i).connectAll();
		}
	}
	
	public void addPropertyChangeListener(PropertyChangeListener listener) {
		if (pcs == null) {
			pcs = new PropertyChangeSupport(this);
		}
		pcs.addPropertyChangeListener(listener);
	}
	public void removePropertyChangeListener(PropertyChangeListener listener) {
		if (pcs != null) {
			pcs.removePropertyChangeListener(listener);
		}
	}
	
	protected void firePropertyChangeEvent(String propertyName, Object oldValue, Object newValue) {
		if (pcs == null || oldValue == newValue || (oldValue != null && oldValue.equals(newValue))) {
			return;
		}
		pcs.firePropertyChange(propertyName, oldValue, newValue);
	}
	
}
