package copeServer.impl.composition;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

/**
 * @author Espen Nersveen
 * Representation class for a service connection.
 */
public class Relation{

	private String description;
	private ComposedService source;
	private ComposedService target;
	private boolean active;
	
	private PropertyChangeSupport pcs;
	
	public Relation(String description, ComposedService source, ComposedService target) {
		this.description = description;
		this.source = source;
		this.target = target;
		this.active = false;
		
		//Notify source and target about the new relation
		source.addRelation(this);
		target.addRelation(this);
	}

//	public void setAttributes(String description, ComposedService source, ComposedService target){
//		
//		this.description = description;
//		this.source = source;
//		this.target = target;
//		this.active = false;
//		
//		//Notify source and target about the new relation
//		source.addRelation(this);
//		target.addRelation(this);
//	}
	
	public boolean isActive() {
		return active;
	}
	
	/**
	 * Activates or deactivates the relation.
	 * @param active
	 */
	public void setActive(boolean active) {
		if(this.active != active){
			if(active){
				source.getService().connect(description, source.getService().getURL(), target.getService().getURL());
				target.getService().connect(description, source.getService().getURL(), target.getService().getURL());
			} else{
				source.getService().disconnect(description, source.getService().getURL(), target.getService().getURL());
				target.getService().disconnect(description, source.getService().getURL(), target.getService().getURL());
			}
			this.active = active;
			firePropertyChangeEvent("relationactivation", new Boolean(!active), new Boolean(active));
		}
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public ComposedService getSource() {
		return source;
	}

	public void setSource(ComposedService source) {
		this.source = source;
	}

	public ComposedService getTarget() {
		return target;
	}

	public void setTarget(ComposedService target) {
		this.target = target;
	}
	
	public void addPropertyChangeListener(PropertyChangeListener listener) {
		if (pcs == null) {
			pcs = new PropertyChangeSupport(this);
		}
		pcs.addPropertyChangeListener(listener);
	}
	
	public void removePropertyChangeListener(PropertyChangeListener listener) {
		if (pcs != null) {
			pcs.removePropertyChangeListener(listener);
		}
	}
	
	protected void firePropertyChangeEvent(String propertyName, Object oldValue, Object newValue) {
		if (pcs == null) {
			return;
		}
		pcs.firePropertyChange(propertyName, oldValue, newValue);
	}
	
	public String toString() {
		return description;
	}
	
}