package copeServer.impl.discovery;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractListModel;

import copeServer.impl.service.Service;
import copeServer.impl.service.ServiceRegistry;

public class DiscoveryManager extends AbstractListModel implements PropertyChangeListener {

	private List discoveryList = new ArrayList();
	
	public DiscoveryManager(){
		AbstractDiscovery manual = new ManualDiscovery();
		AbstractDiscovery rfid = new RFIDDiscovery();
	}
	
	public void addDiscovery(AbstractDiscovery discovery){
		if(!discoveryList.contains(discovery)){
			discoveryList.add(discovery);
			discovery.addPropertyChangeListener(this);
			fireIntervalAdded(this, getSize(), getSize());
			System.out.println("Added new discovery: "+discovery);
		} else{
			System.out.println("ERROR ! Trying to add discovery that is allready there.");
		}
	}
	
	public void removeDiscovery(AbstractDiscovery discovery){
		if(discoveryList.contains(discovery)){
			discoveryList.remove(discovery);
			discovery.removePropertyChangeListener(this);
			fireIntervalRemoved(this, getSize()-1, getSize());
			System.out.println("Removed discovery "+discovery);
		} else{
			System.out.println("ERROR ! Trying to remove discovery that is not there.");
		}
	}
	
	public void propertyChange(PropertyChangeEvent evt) {
		AbstractDiscovery discovery = (AbstractDiscovery)evt.getSource();
		if("url".equals(evt.getPropertyName())){
			int pos = discoveryList.indexOf(discovery);
			if (pos >= 0) {
				System.out.println("Fireing contentsChanged in position: "+pos);
				fireContentsChanged(this, pos, pos);
			} else{
				System.out.println("Error ! Wrong in position of discovery.");
			}
		} else{
			System.out.println("Error ! Property is not [url].");
		}
	}

	public int getSize() {
		return discoveryList.size();
	}

	public Object getElementAt(int pos) {
		return discoveryList.get(pos);
	}
	
}
