package copeServer.impl.discovery;

import gnu.getopt.Getopt;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.TooManyListenersException;
import java.util.Vector;

import javax.comm.CommPortIdentifier;
import javax.comm.NoSuchPortException;
import javax.comm.PortInUseException;
import javax.comm.SerialPort;
import javax.comm.SerialPortEvent;
import javax.comm.SerialPortEventListener;
import javax.comm.UnsupportedCommOperationException;

public class RFIDDiscovery extends AbstractDiscovery implements DiscoveryInterface, SerialPortEventListener{

	private static String lastIDBlue = "";
	private static long lastIDBluedate = 0;
	private boolean debug = false;
	private Thread inTouch;
	/** Create a device driver.
	 */
	public RFIDDiscovery() {
		// init serial port variables
		m_serialPort = null;
		m_serialPortReader = null;
		m_serialPortWriter = null;
		
	}
	/** Open the specified serial port at the given baud rate.
	 */
	public void openSerialPort(String portName, int baudRate) throws NoSuchPortException, PortInUseException, IOException, UnsupportedCommOperationException, TooManyListenersException {
		// close the port if it's open
		closeSerialPort();

		// open the select port by name
		CommPortIdentifier portId = CommPortIdentifier.getPortIdentifier(portName);
		
		m_serialPort = (SerialPort) portId.open(Integer.toString(hashCode()), 2000);
		if (m_serialPort == null) {
			System.err.println("Error opening port " + portId.getName());
			System.exit(1);
		}
	
		m_serialPort.addEventListener(this);
		m_serialPort.notifyOnDataAvailable(true);
		m_serialPort.setFlowControlMode(SerialPort.FLOWCONTROL_NONE);
		m_serialPort.setSerialPortParams(baudRate,
				  SerialPort.DATABITS_8,
				  SerialPort.STOPBITS_1,
				  SerialPort.PARITY_NONE);

		System.out.println("baud rate: " + m_serialPort.getBaudRate());
		System.out.println("data bits: " + m_serialPort.getDataBits());
		System.out.println("stop bits: " + m_serialPort.getStopBits());
		System.out.println("parity:    " + m_serialPort.getParity());
		
		//Start StayInTouchThread to keep the RFID pen alive
		inTouch = new Thread(){
			public void run() {
				try { 
					while(true) {
						sendNoOp();
						if (debug) {
						//	System.out.println("Thread going to sleep....");
						}
						sleep(60000);
					}
				}
				catch(InterruptedException e) {
					e.printStackTrace();
				}
			}
		};
		inTouch.setDaemon(true);
		inTouch.run();
	}

	/** Close serial port and remove the event listener.
	 */
	public void closeSerialPort() {
		// Stop StayInTouchThread
		// inTouch.stop();
		
		if (m_serialPort != null) {
			m_serialPort.close();
			m_serialPort = null;
		}
	}

	/**
	 *  Returns the list of all known serial ports.
	 *
	 *@return String[]   List of ports.
	 */
	public static String[] getPortList() {
		Vector v = new Vector();
		Enumeration portList = CommPortIdentifier.getPortIdentifiers();

		while (portList.hasMoreElements()) {
			CommPortIdentifier portId = (CommPortIdentifier) portList.nextElement();
			//System.out.println(portId.getName());
			if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL) {
				v.add(portId.getName());
			}
		}
		return (String[]) v.toArray(new String[v.size()]);
	}

	/** Handle a serial port event.
	 */
	public void serialEvent(SerialPortEvent event) {
		switch (event.getEventType()) {
			case SerialPortEvent.BI:
			case SerialPortEvent.OE:
			case SerialPortEvent.FE:
			case SerialPortEvent.PE:
			case SerialPortEvent.CD:
			case SerialPortEvent.CTS:
			case SerialPortEvent.DSR:
			case SerialPortEvent.RI:
			case SerialPortEvent.OUTPUT_BUFFER_EMPTY:
				break;
			case SerialPortEvent.DATA_AVAILABLE:
				try {
					int header = -1;
					int l1 = -1;
					int l2 = -1;
					int length = -1; //length of data - 2 bytes
					int[] data = new int[100];
					int crc = -1;

					InputStream m_serialPortReader = m_serialPort.getInputStream();
					header = m_serialPortReader.read();
					l1 = m_serialPortReader.read();
					l2 = m_serialPortReader.read();
					length = (256*l1 + l2);
					//System.out.println("Received header: "+header+", l1: "+l1+", l2 :"+l2+", length: "+length);
					
					for (int i = 0; i < length ; i++){
						data[i] = m_serialPortReader.read();
					}
					crc = m_serialPortReader.read(); //crc check not yet included!
					
					switch (header) {
					case 0x01:
						handleTAG_ID(data, length);
						break;
					case 0x1F:
						handleNACK();
						break;
					case 0xFF:
						handleReadyToRead();
						break; }

				} catch (IOException e) {
					e.printStackTrace();
				}
				break;
		}
	}

	/** Overridden to make sure the serial port is closed before the object is
	 * garbage collected.
	 */
	public void finalize() {
		closeSerialPort();
	}
	
	/** Handle read tag value from RFID
	 * @throws Exception 
	 */
	public void handleTAG_ID(int[] data, int length){
		lastIDBlue = "";
		for (int i = (length - 2) ; i >= 0 ; i--) {
			lastIDBlue += Integer.toHexString(data[i]);
		}
		lastIDBluedate = System.currentTimeMillis();
		
		/*
		 * (non-javadoc)
		 * testing purpose only
		 * don't require rfid-scanning
		 * unquote above for-loop and uncomment one line to make it work
		 */
		//lastIDBlue = "70017a31e4"; //location: king's cross
		//lastIDBlue = "7001bc85a2"; //location: prinsen kino
		//lastIDBlue = "7001246810"; //location: studentersamfundet
		//lastIDBlue = "70018438780"; //user: jose
		//lastIDBlue = "70013g6r54"; //user: carlos
		System.out.println("TagID = "+lastIDBlue);
		lastIDBlue = "http://129.241.219.187/acidmoose/cope/test.jar";
		newURL(lastIDBlue);
	}
	
	/** Handle an IDBlue NACK event
	 */
	protected void handleNACK(){
		if (debug) 
			System.out.println("NACK received");
	}
	
	/** Handle an IDBlue Ready to Read event
	 */
	protected void handleReadyToRead(){
		if (debug) 
			System.out.println("Reader Ready to read RFID");
		sendReadTag();
	}

	
	/** Send packet to IDBlue to initiate RFID tag read
	 */
	protected void sendReadTag() {
		ByteArrayOutputStream buffer = new ByteArrayOutputStream();
		buffer.reset();
		buffer.write(0x01);
		buffer.write(0x00);
		buffer.write(0x00);
		buffer.write(0x01);

		byte[] b = buffer.toByteArray();
		try {
			OutputStream out = m_serialPort.getOutputStream();
			out.write(b);
			//out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/** Send packet to IDBlue to initiate RFID tag read
	 */
	protected void sendNoOp() {
		ByteArrayOutputStream buffer = new ByteArrayOutputStream();
		buffer.reset();
		buffer.write(0x00);
		buffer.write(0x00);
		buffer.write(0x00);
		buffer.write(0x00);

		byte[] b = buffer.toByteArray();
		try {
			OutputStream out = m_serialPort.getOutputStream();
			out.write(b);
			//out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/** Compute a checksum for the given array of bytes.
	 * NB: Old method from another driver.
	 * Not in use - must be adapted to the IDBlue driver
	 */
	public static byte computeChecksum(byte[] bytes) {
		int result = 0;
		for (int i = 0; i < bytes.length; i++)
			result = result ^ bytes[i];
		return (byte) result;
	}

	/** Show the program argument usage and then exit.
	 */
	protected static void printUsageAndExit() {
		System.err.println("usage: IDBlueDriver <serial_port>");
	}

	/**
	 * Activates the IDBlue rfid reader at a given COM port.
	 * Will wait for rfid reader to respond, then wait for reader
	 * to be ready for card, then wait for signal that card is found.
	 * Will also check to see if the tag belongs to a user or a location.
	 * @param argv the COM port which the IDBlue runs on
	 */
	public static RFIDDiscovery activateReader(String argv[]) {
		RFIDDiscovery ret = null;
		// Parse command line options
		int c;
		// - allows return in order functionality
		Getopt g = new Getopt("IDBlueDriver", argv, ":");
		g.setOpterr(true); // do own error handling
		while ((c = g.getopt()) != -1) {
			switch (c) {
				case ':':
					System.err.println("An argument is required for option " + (char) g.getOptopt());
					printUsageAndExit();
					break;
				case '?':
					System.err.println("The option '" + (char) g.getOptopt() + "' is not valid");
					printUsageAndExit();
					break;
			}
		}

		// verify that serial port was specified
		if (g.getOptind() >= argv.length)
			printUsageAndExit();
		String portName = argv[g.getOptind()];
		
		try {
			// Create the driver
			System.out.println("Starting IDBlue Driver on " + portName + "...");
			RFIDDiscovery RFIDdiscovery = new RFIDDiscovery();
			System.out.println("opened driver");
			
			// main operation
			RFIDdiscovery.openSerialPort(portName, 19200);
			System.out.println("opened serialport");
			/*BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
			System.out.println("opened bufferreader");
			System.out.println("Type 'q' to quit.");

			// loop dispatching command strings until we get 'q'
			while (true) {
				String cmd = input.readLine();
				if (cmd.compareToIgnoreCase("q") == 0)
					break;
			}
			IDBlueDriver.closeSerialPort();*/
			ret =  RFIDdiscovery;
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return ret;
		// the exit shuts down rain, otherwise the serive framework continues
		// System.exit(0);
	}

	protected SerialPort m_serialPort;
	protected BufferedReader m_serialPortReader;
	protected BufferedWriter m_serialPortWriter;
	
	public String getLastIDBlue() {
		return lastIDBlue;
	}

	public long getLastIDBluedate() {
		return lastIDBluedate;
	}
	
	public String toString(){
		return "RFID";
	}

	
}
