package copeServer.impl.gui;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import copeServer.impl.composition.Composition;
import copeServer.impl.composition.ComposedService;
import copeServer.impl.composition.Relation;
import copeServer.impl.service.Service;

public class Model implements PropertyChangeListener {
	
	public static final int
	STATE_SANDBOX = 1,
	STATE_DISCOVERY = 2,
	STATE_FAVORITES = 3,
	STATE_MYSERVICES = 4;
	
	private int state = STATE_SANDBOX;
	private Service selectedService = null;
	private Composition selectedComposition = new Composition();
	private ComposedService selectedComposedService = null;
	private Relation selectedRelation = null;
	
	private boolean autoinitiateCompositions = true;
	
	private String statusMessage = "";
	
	PropertyChangeSupport pcs;

	private String icon;
	
	//ADD GUI LOGGING FUNCTIONS HERE !!
	//Make a logger in the model, and change every sysout to write to the log
	//Make the logger a part of this
	//STATUSBAR will read events from this models access to the logger
	
	public Model(){
		
	}
	
	public void setAutoinitiateCompositions(boolean value){
		this.autoinitiateCompositions = value;
	}
	
	public boolean getAutoinitiateCompositions(){
		return autoinitiateCompositions;
	}
	
	public void setMessage(String message){
		String oldMessage = statusMessage;
		statusMessage = message;
		firePropertyChangeEvent("message", oldMessage, statusMessage);
	}
	
	public void moveCompositionToSandbox(){
		firePropertyChangeEvent("composition2sandbox", null, null);
	}
	
	public void moveServiceToSandbox(){
		firePropertyChangeEvent("service2sandbox", null, null);
	}
	
	public void setState(int state){
		int oldState = this.state;
		if(oldState!=state){
			this.state = state;
		} else{
			this.state = this.STATE_SANDBOX;
		}
		firePropertyChangeEvent("state", new Integer(oldState), new Integer(this.state));
	}
	
	public void setSelectedService(Service selectedService){
		Service oldService = this.selectedService;
		this.selectedService = selectedService;
		if(oldService!=null){
			oldService.removePropertyChangeListener(this);
		}
		if(this.selectedService!=null){
			this.selectedService.addPropertyChangeListener(this);
		}
		firePropertyChangeEvent("selectedservice", oldService, this.selectedService);
	}
	
	public void setSelectedComposition(Composition selectedComposition){
		Composition oldComposition = this.selectedComposition;
		this.selectedComposition = selectedComposition;
		firePropertyChangeEvent("selectedcomposition", oldComposition, this.selectedComposition);
	}
	
	public void setSelectedComposedService(ComposedService cs){
		ComposedService oldSelectedComposedService = this.selectedComposedService;
		if(this.selectedRelation!=null)
			setSelectedRelation(null);
		this.selectedComposedService = cs;
		firePropertyChangeEvent("selectedcomposedservice", oldSelectedComposedService, selectedComposedService);
	}
	
	public void setSelectedRelation(Relation rel){
		Relation oldRel = this.selectedRelation;
		if(this.selectedComposedService!=null)
			setSelectedComposedService(null);
		this.selectedRelation = rel;
		firePropertyChangeEvent("selectedrelation", oldRel, selectedRelation);
	}
	
	public Service getSelectedService(){
		return selectedService;
	}
	
	public Composition getSelectedComposition(){
		return selectedComposition;
	}
	
	public ComposedService getSelectedRelation(){
		return selectedComposedService;
	}
	
	public void addPropertyChangeListener(PropertyChangeListener listener) {
		if (pcs == null) {
			pcs = new PropertyChangeSupport(this);
		}
		pcs.addPropertyChangeListener(listener);
	}
	public void removePropertyChangeListener(PropertyChangeListener listener) {
		if (pcs != null) {
			pcs.removePropertyChangeListener(listener);
		}
	}
	
	protected void firePropertyChangeEvent(String propertyName, Object oldValue, Object newValue) {
		if (pcs == null) {
			return;
		}
		pcs.firePropertyChange(propertyName, oldValue, newValue);
	}

	public void propertyChange(PropertyChangeEvent arg0) {
		firePropertyChangeEvent("selectedservice", null, selectedService);
	}

	public String getIcon() {
		return icon;
	}
}
