package copeServer.impl.gui;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.Hashtable;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.SwingUtilities;

import org.jgraph.JGraph;
import org.jgraph.graph.AttributeMap;
import org.jgraph.graph.BasicMarqueeHandler;
import org.jgraph.graph.CellViewRenderer;
import org.jgraph.graph.DefaultCellViewFactory;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.DefaultPort;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.GraphModel;
import org.jgraph.graph.Port;
import org.jgraph.graph.PortRenderer;
import org.jgraph.graph.PortView;

import copeServer.impl.composition.ComposedService;
import copeServer.impl.composition.Composition;
import copeServer.impl.composition.Relation;
import copeServer.impl.ontology.OntologyManager;
import copeServer.impl.service.Service;

public class SandboxPanel extends JGraph implements PropertyChangeListener{

	protected Image bgImage = null;

	private OntologyManager ontologyManager;
	private Composition composition;

	SandboxPanel instance;

	private Model model;

//	private JGraph graph;

	public SandboxPanel(Model model, OntologyManager ontologyManager) {
		this.model = model;
		this.ontologyManager = ontologyManager;
		this.setLayout(new GridLayout(1, 1));
		this.model.addPropertyChangeListener(this);
		this.composition = model.getSelectedComposition();
		this.composition.addPropertyChangeListener(this);

		instance = this;

		createGraphWithNodes();
//		this.add(graph);
//		ToolTipManager.sharedInstance().registerComponent(instance);
	}

	/**
	 * Handles service managing from other views.
	 * 
	 * if("service2sandbox")
	 * 		Converts the service into a relation and adds it to the current Composition.
	 * if("composition2sanbox")
	 * 		Clears the sandbox and adds the selectedcomposition.
	 */
	public void propertyChange(PropertyChangeEvent arg0) {
		if(arg0.getSource()==model){
			if(arg0.getPropertyName().equalsIgnoreCase("service2sandbox")){
				ComposedService relation = new ComposedService(model.getSelectedService());
				model.getSelectedComposition().addRelation(relation);
				insertNode(relation);
			} else if(arg0.getPropertyName().equalsIgnoreCase("composition2sandbox")){
				this.composition.disconnectAll();
				this.composition.removePropertyChangeListener(this);
				for (int i = 0; i < model.getSelectedComposition().getSize(); i++) {
					insertNode(model.getSelectedComposition().getComposedService(i));
				}
				this.composition=model.getSelectedComposition();
				this.composition.addPropertyChangeListener(this);
			}
		} else if(arg0.getSource() instanceof ComposedService){
			if(arg0.getPropertyName().equalsIgnoreCase("state")){
				System.out.println("Rootcount: "+instance.getModel().getRootCount());
			}
		}
	}

	/**
	 * Creats the instance with preferences and sets the mouselistener.
	 */
	private void createGraphWithNodes() {

		// Construct Model and Graph
		GraphModel graphModel = new DefaultGraphModel();

		instance.getGraphLayoutCache().setFactory(new DefaultCellViewFactory() {
			protected PortView createPortView(Object v) {
				if (v instanceof MyPort)
					return new MyPortView(v);
				return super.createPortView(v);
			} 
		});

		instance.setModel(graphModel);

		// Enable Anti Alias
		instance.setAntiAliased(true);
		// Disable auto resize
		instance.setAutoResizeGraph(false);
		// Make Ports Visible by Default
		instance.setPortsVisible(true);
		// Use the Grid (but don't make it Visible)
		instance.setGridEnabled(true);
		// Set the Grid Size to 10 Pixel
		instance.setGridSize(6);
		// Set the Tolerance to 2 Pixel
		instance.setTolerance(2);
		// Accept edits if click on background
		instance.setInvokesStopCellEditing(true);
		// Allows control-drag
		instance.setCloneable(true);
		// Jump to default port on connect
		instance.setJumpToDefaultPort(false);
		// set opaque and load background
		instance.setOpaque(true);
		instance.setBackgroundImage(new ImageIcon("COPE/sandboxback.jpg"));
		// Set a custom marquee handler for displaying a line when connecting
		instance.setMarqueeHandler(new MyMarqueeHandler());
		// Disable the possibility to let users disconnect the edges (using
		// right click for this)
		instance.setConnectable(false);
		instance.setDisconnectable(false);


		// MouseListener that Prints the Cell on Doubleclick
		instance.addMouseListener(new MouseAdapter() {


			public void mouseReleased(MouseEvent e) {
				for (int i = 0; i < instance.getModel().getRootCount(); i++) {
					Object oRoot = instance.getModel().getRootAt(i);

					if (!(oRoot instanceof DefaultEdge)) {
						DefaultGraphCell root = (DefaultGraphCell) oRoot;

						PortView pv = instance.getDefaultPortForCell(root);
						MyPort port = (MyPort) pv.getCell();    

						port.setMode(MyPort.DEFAULT);

						instance.getGraphLayoutCache().edit(new Object[]{port}, port.getAttributes());
					}
				}
			}

			public void mousePressed(MouseEvent e) {

				// If left click
				if (e.getButton() == MouseEvent.BUTTON1) {
					//Get Cell under Mousepointer
					int x = e.getX(), y = e.getY();
					Object cell = instance.getFirstCellForLocation(x, y);
					if(cell!=null){
						if (cell instanceof DefaultEdge) {
							DefaultEdge edge = (DefaultEdge) cell;
							model.setSelectedRelation((Relation)edge.getUserObject());
						} else if (cell instanceof DefaultGraphCell) {
							DefaultGraphCell vertex = (DefaultGraphCell) cell;
							model.setSelectedComposedService((ComposedService) vertex.getUserObject());
						}
					} else{
						model.setSelectedComposedService(null);
						model.setSelectedRelation(null);
					}
					// If double click
					if (e.getClickCount() == 2) {
						// Print Cell Label (do your stuff here)
						if (cell != null && !(cell instanceof DefaultEdge)) {
							((ComposedService)((DefaultGraphCell)cell).getUserObject()).getService().openInterface();
						}
					} else{
						PortView p = instance.getPortViewAt(e.getPoint().getX(), e.getPoint().getY());
						if(p!=null){
							Service sourceService = ((ComposedService)((DefaultGraphCell)cell).getUserObject()).getService();
							for (int i = 0; i < instance.getModel().getRootCount(); i++) {

								Object oRoot = instance.getModel().getRootAt(i);
								if (!(oRoot instanceof DefaultEdge) && oRoot instanceof DefaultGraphCell) {
									
									DefaultGraphCell root = (DefaultGraphCell) oRoot;

									ComposedService targetComposedService = (ComposedService) root.getUserObject();
									Service targetService = targetComposedService.getService();

									PortView pv = instance.getDefaultPortForCell(root);
									MyPort port = (MyPort) pv.getCell();         
									if(sourceService == targetService) {
										port.setMode(MyPort.DEFAULT);
										instance.getGraphLayoutCache().edit(new Object[]{port}, port.getAttributes());
										
									} else if(ontologyManager.match(sourceService, targetService)){
										System.out.println(sourceService+", "+targetService+" matches");

										port.setMode(MyPort.OK);
										instance.getGraphLayoutCache().edit(new Object[]{port}, port.getAttributes());
									} else{
										System.out.println(sourceService+", "+targetService+" does not match");

										port.setMode(MyPort.NOK);
										instance.getGraphLayoutCache().edit(new Object[]{port}, port.getAttributes());
									}
								}
							}
						}
					}
					// If right click
				} else if (e.getButton() == MouseEvent.BUTTON3) {
					// Get edge under Mousepointer
					int x = e.getX(), y = e.getY();
					Object cell = instance.getFirstCellForLocation(x, y);
					if (cell != null && cell instanceof DefaultEdge) {
						DefaultEdge edge = (DefaultEdge) cell;
//						ComposedService source = ((ComposedService)((DefaultGraphCell)instance.getModel().getParent(edge.getSource())).getUserObject());
//						ComposedService target = ((ComposedService)((DefaultGraphCell)instance.getModel().getParent(edge.getTarget())).getUserObject());
//						Relation relation = (Relation) ((DefaultGraphCell)instance.getModel().getParent(edge)).getUserObject();
						Relation relation = (Relation) edge.getUserObject();
//						source.disconnect(target.getService());
//						source.removePropertyChangeListener(instance);
						relation.setActive(false);
						instance.getGraphLayoutCache().remove(new DefaultEdge[] { edge });
					}
				}
			}
		});

	}

	/**
	 * Insert a new Relation (finds icon before inserting)
	 * @param relation
	 */
	private void insertNode(ComposedService relation){

		String icon = relation.getService().getIconPath();
		File iconFile = new File(icon);
		if(icon==null || !iconFile.exists()){
			icon = "COPE/DefaultServiceIcon.png";
		}
		instance.getGraphLayoutCache().insert(createVertex(relation, icon));
		relation.addPropertyChangeListener(this);
	}

	/**
	 * @param userObject
	 * @param icon
	 * @return
	 */
	private DefaultGraphCell createVertex(ComposedService userObject, String icon) {

		DefaultGraphCell vertex = new DefaultGraphCell();

		if (userObject != null)
			vertex.setUserObject(userObject);

		int x = (int) (Math.random() * 100), y = (int) (Math.random() * 100);

		if(userObject.getX()!=null){
			x = Integer.parseInt(userObject.getX());
		} else{
			userObject.setX(Integer.toString(x));
		}
		if(userObject.getY()!=null){
			y = Integer.parseInt(userObject.getY());
		} else{
			userObject.setY(Integer.toString(y));
		}

		// Set different attributes for the cell
		GraphConstants.setBounds(vertex.getAttributes(), new Rectangle(
				new Point((int) x, (int) y)));
		GraphConstants.setSizeable(vertex.getAttributes(), false);
		GraphConstants.setEditable(vertex.getAttributes(), false);
		GraphConstants.setAutoSize(vertex.getAttributes(), true);
		GraphConstants.setOpaque(vertex.getAttributes(), false);
		GraphConstants.setFont(vertex.getAttributes(), new Font("Arial", Font.PLAIN, 10));

		ImageIcon img = new ImageIcon(Toolkit.getDefaultToolkit().getImage(icon).getScaledInstance(48, 48, Image.SCALE_SMOOTH));

		GraphConstants.setIcon(vertex.getAttributes(), img);

		MyPort myPort = new MyPort();
		Map portMap = new Hashtable();
		GraphConstants.setOffset(portMap, new Point2D.Double(750, 700));
		myPort.getAttributes().applyMap(portMap);

		vertex.add(myPort);

		return vertex;
	}


	/**
	 * Insert a new Edge between source and target
	 * 
	 * @param source
	 * @param target
	 */
	public void connect(Port source, Port target) {
		// Construct Edge with no label
		DefaultEdge edge = new DefaultEdge();

		if (instance.getModel().acceptsSource(edge, source) && instance.getModel().acceptsTarget(edge, target)) {

			// Add a Line End Attribute
			GraphConstants.setLineEnd(edge.getAttributes(),
					GraphConstants.ARROW_SIMPLE);
			// Add a label along edge attribute
			GraphConstants.setLabelAlongEdge(edge.getAttributes(), true);
			// Making the edge non-selectable and non-editable
			GraphConstants.setEditable(edge.getAttributes(), false);
			GraphConstants.setSizeable(edge.getAttributes(), false);

//			GraphConstants.setRouting(edge.getAttributes(), ???);
//			GraphConstants.setRemoveAttributes(edge.getAttributes(), new Object[] { GraphConstants.POINTS }); 

			// Insert the Edge and its Attributes
			instance.getGraphLayoutCache().insertEdge(edge, source, target);
			System.out.println("connect("+edge+", "+source+", "+target+")");

			ComposedService composedServiceSource = (ComposedService) ((DefaultGraphCell)instance.getModel().getParent(source)).getUserObject();
			ComposedService composedServiceTarget = (ComposedService) ((DefaultGraphCell)instance.getModel().getParent(target)).getUserObject();

			Relation relation = new Relation(ontologyManager.getDescriptionFromConnection(composedServiceSource.getService(), composedServiceTarget.getService()), composedServiceSource, composedServiceTarget);

			//composedServiceSource.setConnection(ontologyManager.getDescriptionFromConnection(composedServiceSource.getService(), composedServiceTarget.getService()), composedServiceTarget.getService().getURL(), false);
			if(model.getAutoinitiateCompositions())
				relation.setActive(true);

			edge.setUserObject(relation);
		}
	}


	/***********************************************************************************
	 * 
	 * Some overriding and extending classes and method for making my own ports
	 * 
	 ************/
	private class MyPort extends DefaultPort{

		protected static final int DEFAULT = 0;

		protected static final int OK = 1;

		protected static final int NOK = 2;

		int mode = 0;

		String image;

		// Empty Constructor
		public MyPort() {
			this(null);
			setMode(DEFAULT);
		}

		public void setMode(int mode) {
			switch (mode) {
			case DEFAULT:
				image = "COPE/connect_24.png";
				break;
			case OK:
				image = "COPE/ok_24.png";
				break;
			case NOK:
				image = "COPE/nok_24.png";
				break;
			default:
				image = "COPE/connect_24.png";
				break;
			}
		}
		// Construct Cell for Userobject
		public MyPort(Object userObject) {
			super(userObject);
		}

		public void setImage(String image) {
			this.image = image;
		}

		public String getImage() {
			return image;
		}
	} 

	public class MyPortView extends PortView
	{
		private MyPortRenderer renderer;

		public MyPortView(Object cell) {
			super(cell);

			MyPort obj = (MyPort)cell;
			renderer = new MyPortRenderer(obj.getImage());   
		}

		/**
		 * Returns the bounds for the port view.
		 */
		public Rectangle2D getBounds() {
			Point2D pt = (Point2D) getLocation(null).clone();
			int width = 24;
			int height = 24;
			Rectangle2D bounds = new Rectangle2D.Double();

			// TODO denne linjen srger for at det blir ramme rundt porten...
			bounds.setFrame(pt.getX() - width / 2, pt.getY() - height / 2, width, height);
			return bounds;
		}

		public CellViewRenderer getRenderer() {
			return renderer;
		}
	}


	public class MyPortRenderer extends PortRenderer implements CellViewRenderer
	{

		public MyPortRenderer(String image) {
			super();      
		}

		public void paint(Graphics g) {
			if(preview) {
				//TODO skal noe gjres her ?
			}

			MyPort myPort = ((MyPort) this.view.getCell());

			g.setColor(instance.getBackground());
			// Set Xor-Mode Color
			g.setXORMode(instance.getBackground());

			ImageIcon portIcon = new ImageIcon(Toolkit.getDefaultToolkit().createImage(myPort.image));
			portIcon.paintIcon(instance, g, 0, 0);

//			if(myPort.mode != 0) {
//			g.setColor(instance.getBackground());

//			ImageIcon portIcon = new ImageIcon(Toolkit.getDefaultToolkit().createImage(myPort.image));
//			portIcon.paintIcon(instance, g, 0, 0);

//			} else {
//			super.paint(g);
//			}
		}
	}

	/***************************************************************************
	 * 
	 * MyMarqueeHandler
	 * 
	 **************************************************************************/
	//
	// Custom MarqueeHandler
	// MarqueeHandler that Connects Vertices and Displays PopupMenus
	public class MyMarqueeHandler extends BasicMarqueeHandler{

		// Holds the Start and the Current Point
		protected Point2D start, current;

		// Holds the First and the Current Port
		protected PortView port, firstPort;

		// Override to Gain Control (for PopupMenu and ConnectMode)
		public boolean isForceMarqueeEvent(MouseEvent e) {
			if (e.isShiftDown())
				return false;
			// If Right Mouse Button we want to Display the PopupMenu
			if (SwingUtilities.isRightMouseButton(e))
				// Return Immediately
				return true;
			// Find and Remember Port
			port = getSourcePortAt(e.getPoint());
			// If Port Found and in ConnectMode (=Ports Visible)
			if (port != null && instance.isPortsVisible())
				return true;
			// Else Call Superclass
			return super.isForceMarqueeEvent(e);
		}

		// Show Special Cursor if Over Port
		public void mouseMoved(MouseEvent e) {
//			paintPort(instance.getGraphics());

			// Check Mode and Find Port
			if (e != null && getSourcePortAt(e.getPoint()) != null && instance.isPortsVisible()) {
				// Set Cusor on Graph (Automatically Reset)
				instance.setCursor(new Cursor(Cursor.HAND_CURSOR));
				// Consume Event
				// Note: This is to signal the BasicGraphUI's
				// MouseHandle to stop further event processing.
				e.consume();
			} else
				// Call Superclass
				super.mouseMoved(e);
		}

		// Display PopupMenu or Remember Start Location and First Port
		public void mousePressed(final MouseEvent e) {
//			paintPort(instance.getGraphics());
			// If Right Mouse Button
			if (SwingUtilities.isRightMouseButton(e)) {
				// Find Cell in Model Coordinates
				Object cell = instance.getFirstCellForLocation(e.getX(), e.getY());

				// Else if in ConnectMode and Remembered Port is Valid
			} else if (port != null && instance.isPortsVisible()) {
				// Remember Start Location
				start = instance.toScreen(port.getLocation());
				// Remember First Port
				firstPort = port;
			} else {
				// Call Superclass
				super.mousePressed(e);
			}
		}

		// Find Port under Mouse and Repaint Connector
		public void mouseDragged(MouseEvent e) {
//			paintPort(instance.getGraphics());
			// If remembered Start Point is Valid
			if (start != null) {
				// Fetch Graphics from Graph
				Graphics g = instance.getGraphics();
				// Reset Remembered Port
				PortView newPort = getTargetPortAt(e.getPoint());
				// Do not flicker (repaint only on real changes)
				if (newPort == null || newPort != port) {
					// Xor-Paint the old Connector (Hide old Connector)
					paintConnector(Color.black, instance.getBackground(), g);
					// If Port was found then Point to Port Location
					port = newPort;
					if (port != null)
						current = instance.toScreen(port.getLocation());

					// Else If no Port was found then Point to Mouse Location
					else
						current = instance.snap(e.getPoint());

					// Xor-Paint the new Connector
					paintConnector(instance.getBackground(), Color.black, g);

				}
			}
			if(port != null)
				System.out.println("Currently pointing at " + ((DefaultGraphCell)((MyPort)port.getCell()).getParent()).getUserObject());
			// Call Superclass
			super.mouseDragged(e);
		}

		// Connect the First Port and the Current Port in the Graph or Repaints
		public void mouseReleased(MouseEvent e) {
//			paintPort(instance.getGraphics());
			// If Valid Event, Current and First Port
			if (e != null && port != null && firstPort != null && firstPort != port && firstPort.getParentView() != port.getParentView()) {
				// Then Establish Connection
				connect((Port) firstPort.getCell(), (Port) port.getCell());
				e.consume();
			// Else Repaint the Graph
			} else
				instance.repaint();
			
			// Reset Global Vars
			firstPort = port = null;
			start = current = null;
			// Call Superclass
			super.mouseReleased(e);
		}

		public PortView getSourcePortAt(Point2D point) {
			// Disable jumping
			instance.setJumpToDefaultPort(false);
			PortView result;
			try {
				// Find a Port View in Model Coordinates and Remember
				result = instance.getPortViewAt(point.getX(), point.getY());
			} finally {
				instance.setJumpToDefaultPort(false);
			}
			return result;
		}

		// Find a Cell at point and Return its first Port as a PortView
		protected PortView getTargetPortAt(Point2D point) {
			// Find a Port View in Model Coordinates and Remember
			return instance.getPortViewAt(point.getX(), point.getY());
		}

		// Use Xor-Mode on Graphics to Paint Connector
		protected void paintConnector(Color fg, Color bg, Graphics g) {
			// Set Foreground
			g.setColor(fg);
			// Set Xor-Mode Color
			g.setXORMode(bg);
			// Highlight the Current Port
//			paintPort(instance.getGraphics());
			// If Valid First Port, Start and Current Point
			if (firstPort != null && start != null && current != null) {
				// Then Draw A Line From Start to Current Point
				g.drawLine((int) start.getX(), (int) start.getY(), (int) current.getX(), (int) current.getY());
			}
		}

		/*
		 * XXX This one is not in use after i added the Custom Port, Custom Port View and Custom Port Renderer
		 */
		// Use the Preview Flag to Draw a Highlighted Port
		protected void paintPort(Graphics g) {
			// If Current Port is Valid
			if (port != null) {
				// If Not Floating Port...
				boolean o = (GraphConstants.getOffset(port.getAllAttributes()) != null);
				// ...Then use Parent's Bounds
				Rectangle2D r = (o) ? port.getBounds() : port.getParentView().getBounds();
				// Scale from Model to Screen
				r = instance.toScreen((Rectangle2D) r.clone());
				// Add Space For the Highlight Border
				r.setFrame(r.getX() - 3, r.getY() - 3, r.getWidth() + 6, r.getHeight() + 6);
				// Paint Port in Preview (=Highlight) Mode
				instance.getUI().paintCell(g, port, r, true);
			}
		}

	} // End of MyMarqueeHandler



}
