package copeServer.impl.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;

import copeServer.impl.composition.CompositionRegistry;
import copeServer.impl.device.Device;
import copeServer.impl.discovery.DiscoveryManager;
import copeServer.impl.ontology.OntologyManager;
import copeServer.impl.service.ServiceHandler;
import copeServer.impl.service.ServiceRegistry;

public class WindowManager extends JFrame implements PropertyChangeListener{

	private Device device;
	private ServiceRegistry serviceRegistry;
	private ServiceHandler serviceHandler;
	private DiscoveryManager discoveryManager;
	private OntologyManager ontologyManager;
	private CompositionRegistry compositionRegistry;
	
	private SandboxPanel sandbox;
	private SandboxButtonPanel sandboxbp;
	
	private DiscoveryPanel discovery;
	private DiscoveryButtonPanel discoverybp;
	
	private FavoritesPanel favorites;
	private FavoritesButtonPanel favoritesbp;
	
	private ServicesPanel services;
	private ServicesButtonPanel servicesbp;
	
	private StatusBar statusbar;
	
	private MenuPanel menupanel;
	
	private JPanel mainPanel;
	private JPanel contentPanel;
	private JPanel buttonContentPanel;
	
	private Model model;
	
	public WindowManager(Device device, ServiceRegistry serviceRegistry, ServiceHandler serviceHandler, DiscoveryManager discoveryManager, OntologyManager ontologyManager, CompositionRegistry compositionRegistry) {
		super("CoPE");
		
		//Setting models
		this.model = new Model();
		this.device = device;
		this.serviceRegistry = serviceRegistry;
		this.serviceHandler = serviceHandler;
		this.discoveryManager = discoveryManager;
		this.ontologyManager = ontologyManager;
		this.compositionRegistry = compositionRegistry;
		
		init();
		
		//Showing JFrame
		this.setContentPane(mainPanel);
		this.pack();
		this.setVisible(true);
	}
	
	public void init(){
		//Initiating panels
		sandboxbp = new SandboxButtonPanel(model, compositionRegistry);
		sandbox = new SandboxPanel(model, ontologyManager);

		discoverybp = new DiscoveryButtonPanel(model, serviceHandler);
		discovery = new DiscoveryPanel(model, discoveryManager, serviceHandler);

		favoritesbp = new FavoritesButtonPanel(model, compositionRegistry);
		favorites = new FavoritesPanel(model, compositionRegistry);

		servicesbp = new ServicesButtonPanel(model);
		services = new ServicesPanel(model, serviceRegistry);
		
		statusbar = new StatusBar(model);
		
		menupanel = new MenuPanel(model);
		
		mainPanel = new JPanel();
		contentPanel = new JPanel();
		buttonContentPanel = new JPanel();

		//Setting size and attributes of GUI
		mainPanel.setBackground(Color.WHITE);
		mainPanel.setPreferredSize(new Dimension(device.getWidth(), device.getHeight()));
		menupanel.setPreferredSize(new Dimension(device.getWidth(), device.getHeight()/10));
//		buttonContentPanel.setPreferredSize(new Dimension(device.getWidth(), device.getHeight()/5));
		statusbar.setPreferredSize(new Dimension(device.getWidth(), device.getHeight()/50));
//		int tempheight = (int) (menupanel.getPreferredSize().getHeight()-buttonContentPanel.getPreferredSize().getHeight()-statusbar.getPreferredSize().getHeight());
//		contentPanel.setPreferredSize(new Dimension(device.getWidth(), device.getHeight()-tempheight));
		
		//NB! NB! NB!
		//FOLLOWING 8 LINES SHOULD BE CALCULATED FROM DEVICE INFORMATION
		sandbox.setPreferredSize(new Dimension(400, 325));
		discovery.setPreferredSize(new Dimension(400, 325));
		favorites.setPreferredSize(new Dimension(400, 325));
		services.setPreferredSize(new Dimension(400, 325));
		sandboxbp.setPreferredSize(new Dimension(400, 110));
		discoverybp.setPreferredSize(new Dimension(400, 110));
		favoritesbp.setPreferredSize(new Dimension(400, 110));
		servicesbp.setPreferredSize(new Dimension(400, 110));
		
		menupanel.setBackground(Color.WHITE);
		sandboxbp.setBackground(Color.WHITE);
		discoverybp.setBackground(Color.WHITE);
		favoritesbp.setBackground(Color.WHITE);
		servicesbp.setBackground(Color.WHITE);
		statusbar.setBackground(Color.BLUE);
		buttonContentPanel.setBackground(Color.WHITE);
		contentPanel.setBackground(Color.WHITE);
		
		
		contentPanel.add(sandbox);
		buttonContentPanel.add(sandboxbp);
		
		//Setting panel structure
		mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.PAGE_AXIS));
		mainPanel.add(menupanel);
		mainPanel.add(contentPanel);
		mainPanel.add(buttonContentPanel);
		mainPanel.add(statusbar);
		
		//Setting listeners
		model.addPropertyChangeListener(this);
	}

	public void propertyChange(PropertyChangeEvent arg0) {
		if(arg0.getPropertyName().equalsIgnoreCase("state")){
			contentPanel.removeAll();
			buttonContentPanel.removeAll();
			Integer temp = (Integer)arg0.getNewValue();
			int newValue = temp.intValue();
			temp = (Integer)arg0.getOldValue();
			if(newValue==Model.STATE_DISCOVERY){
				contentPanel.add(discovery);
				buttonContentPanel.add(discoverybp);
			} else if(newValue==Model.STATE_FAVORITES){
				contentPanel.add(favorites);
				buttonContentPanel.add(favoritesbp);
			} else if(newValue==Model.STATE_MYSERVICES){
				contentPanel.add(services);
				buttonContentPanel.add(servicesbp);
			} else{
				contentPanel.add(sandbox);
				buttonContentPanel.add(sandboxbp);
			}
			contentPanel.updateUI();
			buttonContentPanel.updateUI();
		}
	}
	
}
