package copeServer.impl.ontology;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;

import copeServer.impl.service.Service;
import copeServer.impl.service.ServiceRegistry;

public class OntologyManager implements OntologyManagerInterface {

	Hashtable serviceMatches = new Hashtable();

	private ServiceRegistry serviceRegistry;

	public OntologyManager(ServiceRegistry serviceRegistry) {
		this.serviceRegistry = serviceRegistry;
	}

	public void generateMatches(Service service) {
		System.out.println("OntologyManager.generateMatches: " + service);
		List matches;
		
		Enumeration serviceKeys = service.getDescription().keys();
		while (serviceKeys.hasMoreElements()) {
			String serviceKey = (String) serviceKeys.nextElement();
			if(serviceKey.startsWith("connection.originating")) {
				for (int i = 0; i < serviceRegistry.getSize(); i++) {
					Service otherService = (Service) serviceRegistry.getElementAt(i);
					Enumeration otherServiceKeys = otherService.getDescription().keys();
					while (otherServiceKeys.hasMoreElements()) {
						String otherServiceKey = (String) otherServiceKeys.nextElement();
						if(otherServiceKey.startsWith("connection.terminating")) {
							String serviceKeyValue = service.getDescription().getProperty(serviceKey);
							String otherServiceKeyValue = otherService.getDescription().getProperty(otherServiceKey);
							if(serviceKeyValue.equalsIgnoreCase(otherServiceKeyValue)) {
								if(serviceMatches.containsKey(service)) {
									matches = (List) serviceMatches.get(service);
								} else {
									matches = new ArrayList();
								}
								matches.add(new Match(otherService, otherServiceKeyValue));
								serviceMatches.put(service, matches);
							}
						}
					}
				}
			} else if(serviceKey.startsWith("connection.terminating")) {
				for (int i = 0; i < serviceRegistry.getSize(); i++) {
					Service otherService = (Service) serviceRegistry.getElementAt(i);
					Enumeration otherServiceKeys = otherService.getDescription().keys();
					while (otherServiceKeys.hasMoreElements()) {
						String otherServiceKey = (String) otherServiceKeys.nextElement();
						if(otherServiceKey.startsWith("connection.originating")) {
							String serviceKeyValue = service.getDescription().getProperty(serviceKey);
							String otherServiceKeyValue = otherService.getDescription().getProperty(otherServiceKey);
							if(serviceKeyValue.equalsIgnoreCase(otherServiceKeyValue)) {
								if(serviceMatches.containsKey(otherService)) {
									matches = (List) serviceMatches.get(otherService);
								} else {
									matches = new ArrayList();
								}
								matches.add(new Match(service, serviceKeyValue));
								serviceMatches.put(otherService, matches);
							}
						}
					}
				}
			}
		}
	}

	public String toString() {
		String result = "\n\n********** PRINTING MATCHES **********\n";
		Enumeration serviceMatchesKeys = serviceMatches.keys();
		while (serviceMatchesKeys.hasMoreElements()) {
			Service service = (Service) serviceMatchesKeys.nextElement();
			result += "Service " + service.getName() + " has the following matches:\n";
			List matches = (List) serviceMatches.get(service);
			for (int i = 0; i < matches.size(); i++) {
				Match match = (Match) matches.get(i);
				result += "\t" + match.service.getName() + " through " + match.connection + "\n";
			}
			if(matches.size()==0)
				result += "\tno matches...\n";
		}
		result += "********** PRINTING    DONE **********\n\n";
		return result;
	}

	public boolean match(Service source, Service target) {
		
		if (source.equals(target))
			return false;

		if (serviceMatches.containsKey(source)) {
			List matches = (List) serviceMatches.get(source);
			for (int i = 0; i < matches.size(); i++) {
				if(((Match)matches.get(i)).service == target)
					return true;
			}
		} else {
			System.out.println("The source (" + source.getName() + ") has no generated matches!!");
		}
		return false;
	}

	public String getDescriptionFromConnection(Service source, Service target) {
		System.out.println("getDescriptionFromConnection called" );
		if (serviceMatches.containsKey(source)) {
			List matches = (List) serviceMatches.get(source);
			for (int i = 0; i < matches.size(); i++) {
				if(((Match)matches.get(i)).service == target) {
					System.out.println("description is " + ((Match)matches.get(i)).connection);
					return ((Match)matches.get(i)).connection;
				}
			}
		}
		System.out.println("no description found");		
		return "";
	}
	
	private class Match {
		Service service;
		String connection;
		public Match(Service service, String connection) {
			super();
			this.service = service;
			this.connection = connection;
		}
		
	}
}