package copeServer.impl.service;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import java.util.List;
import java.util.Properties;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleException;

import copeServer.impl.composition.Relation;
import copeServer.impl.connection.SOAPServiceManaged;
import copeServer.impl.connection.ServiceConnectionInterface;
import copeServer.impl.device.Device;

public class Service implements ServiceConnectionInterface{

	private Bundle bundle;
	private String url;
	private long id;
	private String name;
	private int state;
	private String icon;
	private ServiceConnectionInterface serviceManaged;
	
	public static final int CONNECTION_SOAP = 1, CONNECTION_BLUETOOTH = 2;
	
	PropertyChangeSupport pcs;
	
	public Service(Bundle bundle, int connection){
		this.bundle = bundle;
		this.id = bundle.getBundleId();
	}

	public long getID(){
		return id;
	}
	
	/*
	 * Methods for property change management
	 */
	public void addPropertyChangeListener(PropertyChangeListener listener) {
		if (pcs == null) {
			pcs = new PropertyChangeSupport(this);
		}
		pcs.addPropertyChangeListener(listener);
	}
	public void removePropertyChangeListener(PropertyChangeListener listener) {
		if (pcs != null) {
			pcs.removePropertyChangeListener(listener);
		}
	}
	
	protected void firePropertyChangeEvent(String propertyName, Object oldValue, Object newValue) {
		if (pcs == null || oldValue == newValue || (oldValue != null && oldValue.equals(newValue))) {
			return;
		}
		pcs.firePropertyChange(propertyName, oldValue, newValue);
	}
	
	public String getName() {
		return name;
	}
	
	public void handshake(){
		String SOAPName = (String) bundle.getRegisteredServices()[0].getProperty("SOAP.service.name");
		serviceManaged = new SOAPServiceManaged(SOAPName);
		serviceManaged.setID(id);
		
		this.name = serviceManaged.getName();
		this.url = serviceManaged.getURL();
		this.state = bundle.getState();
		this.icon = serviceManaged.getIconPath();
		
	}

	public void setName() {
		
	}

	public void unregister() {
		try {
			firePropertyChangeEvent("uninstalled", null, null);
			bundle.uninstall();
		} catch (BundleException e) {
			e.printStackTrace();
		}
	}
	
	public void start(){
		try {
			bundle.start();
		} catch (BundleException e) {
			e.printStackTrace();
		}
	}
	
	public void stop(){
		try {
			bundle.stop();
		} catch (BundleException e) {
			e.printStackTrace();
		}
	}
	
	public void setState(int state) {
		int oldState = this.state;
		this.state = state;
		firePropertyChangeEvent("state", Integer.toString(oldState), Integer.toString(state));
	}
	
	public int getState(){
		return this.state;
	}
	
	public String toString(){
		String ret = name;
		return ret;
	}

	public void setURL(String url) {
		serviceManaged.setURL(url);
	}

	public void ping() {
		serviceManaged.ping();
	}

	public void connect(String description, String source, String target) {
		serviceManaged.connect(description, source, target);
	}
	
	public void disconnect(String description, String source, String target) {
		serviceManaged.disconnect(description, source, target);
	}

	public void setID(long id) {
		serviceManaged.setID(id);
	}

	public void event(String description, Object oldValue, Object newValue) {
		serviceManaged.event(description, oldValue, newValue);
	}

	public Properties getDescription() {
		return serviceManaged.getDescription();
	}

	public void openInterface() {
		serviceManaged.openInterface();
	}

	public String getURL() {
		return url;
	}

	public void setURL() {
		serviceManaged.getURL();
	}
	
	public String getIconPath() {
		return icon;
	}



}
