package copeServer.impl.service;

import java.io.File;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleException;

import copeServer.impl.composition.Composition;
import copeServer.impl.composition.CompositionRegistry;
import copeServer.impl.composition.ComposedService;
import copeServer.impl.ontology.OntologyManager;
import copeServer.impl.tools.XMLManager;

public class ServiceHandler {
	
	private ServiceRegistry serviceRegistry;
	private BundleContext bundleContext;
	private CompositionRegistry compositionRegistry;
	
	File sessionfile = new File("COPE/session.xml");
	OntologyManager ontologyManager;
	
	public ServiceHandler(ServiceRegistry serviceRegistry, CompositionRegistry compositionRegistry, OntologyManager ontologyManager, BundleContext context) {
		this.serviceRegistry = serviceRegistry;
		this.compositionRegistry = compositionRegistry;
		this.ontologyManager = ontologyManager;
		this.bundleContext = context;
		
		
		//Loading previous session
		if(sessionfile.exists()){
			System.out.println("sessionfile exists");
			XMLManager man = new XMLManager(sessionfile.getPath(), false);
			List services = man.getServiceList();
			for (int i = 0; i < services.size(); i++) {
//				reRegisterService(Long.getLong((String)((Hashtable)services.get(i)).get("id")));
				reRegisterService(Long.parseLong((String) ((Hashtable)services.get(i)).get("id")));
			}
		}
	}
	
	public ServiceRegistry getServiceRegistry(){
		return serviceRegistry;
	}
	
	public void installService(String url){
		try {
			Bundle bundle = bundleContext.installBundle(url);
			System.out.println("Adding bundle");
			serviceRegistry.addService(new Service(bundle, Service.CONNECTION_SOAP));
			System.out.println("Starting bundle");
			serviceRegistry.getService(bundle.getBundleId()).start();
			System.out.println("Performing handshake");
			serviceRegistry.getService(bundle.getBundleId()).handshake();
			ontologyManager.generateMatches(serviceRegistry.getService(bundle.getBundleId()));
		} catch (BundleException e) {
			e.printStackTrace();
		}
	}
	
	public void reRegisterService(long bid){
		System.out.println("Reregistering bundle");
		Service service = new Service(bundleContext.getBundle(bid), Service.CONNECTION_SOAP);
		serviceRegistry.addService(service);
		System.out.println("Reperforming handshake");
		service.handshake();
		ontologyManager.generateMatches(service);
	}

	public void saveAll() {
		ArrayList servicesList = new ArrayList();
		for (int i = 0; i < serviceRegistry.getSize(); i++) {
			Service service = (Service)serviceRegistry.getElementAt(i);
			Hashtable table = new Hashtable();
			table.put("id", Long.toString(service.getID()));
			servicesList.add(table);
		}
		
		ArrayList compositionsList = new ArrayList();
		ArrayList relationsList = new ArrayList();
		Composition composition;
		for (int i = 0; i < compositionRegistry.getSize(); i++) {
			relationsList = new ArrayList();
			composition = (Composition) compositionRegistry.getElementAt(i);
			ComposedService relation;
			Hashtable table;
			for (int j = 0; j < composition.getSize(); j++) {
				table = new Hashtable();
				relation = composition.getComposedService(i);
				table.put("origin", Long.toString(relation.getService().getID()));
//				table.put("connected", relation.getURLs());
				table.put("x", relation.getX());
				table.put("y", relation.getY());
				relationsList.add(table);
			}
			compositionsList.add(relationsList);
		}
		
		XMLManager man = new XMLManager("COPE/session.xml", false);
		man.save(servicesList, compositionsList);
	}

}
