package copeServer.impl.tools;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class XMLManager {
	
	private Document doc;
	private String filename;
	private boolean validating;
	
	private List serviceList = new ArrayList();
	private List compositionList = new ArrayList();
	
	public XMLManager(String filename, boolean validating) {
		this.filename = filename;
		this.validating = validating;
		
	}
	
	private void init(){
		try {
			//Create a builder factory
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			factory.setValidating(validating);
			
			// Create the builder and parse the file
			doc = factory.newDocumentBuilder().parse(new File(filename));
			doc.getDocumentElement().normalize();
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * @return Services (ArrayList -HashTable-) with values: "id".
	 */
	public List getServiceList(){
		init();
		NodeList list = doc.getElementsByTagName("bundle");
		for (int i = 0; i < list.getLength(); i++) {
			Hashtable table = new Hashtable();
			Node serviceNode = list.item(i);
			if(serviceNode.getNodeType() == Node.ELEMENT_NODE){
				Element serviceElement = (Element)serviceNode;
				NodeList idList = serviceElement.getElementsByTagName("id");
				Element firstNameElement = (Element)idList.item(0);
				NodeList textFNList = firstNameElement.getChildNodes();
				table.put("id", ((Node)textFNList.item(0)).getNodeValue().trim());
//				idList = serviceElement.getElementsByTagName("soapname");
//				firstNameElement = (Element)idList.item(0);
//				textFNList = firstNameElement.getChildNodes();
//				table.put("soapname", ((Node)textFNList.item(0)).getNodeValue().trim());
				serviceList.add(table);
			}
		}
		return serviceList;
	}
	
	/**
	 * @return Compositions (ArrayList) containing Relations (ArrayList -HashTable-). Values in the hashtable is: "origin", "connected", "x", "y".
	 */
	public List getCompositionList(){
		init();
		//Getting compositions from document
		NodeList list = doc.getElementsByTagName("composition");
		//Traversing compositions
		for (int i = 0; i < list.getLength(); i++) {
			Node compositionNode = list.item(i);
			if(compositionNode.getNodeType() == Node.ELEMENT_NODE){
				//Getting relations from composition
				ArrayList relations = new ArrayList();
				Element compositionElement = (Element)compositionNode;
				NodeList relationList = compositionElement.getElementsByTagName("relation");
				
				//Traversing relations
				for (int j = 0; j < relationList.getLength(); j++) {
					Node relationNode = relationList.item(j);
					Hashtable table = new Hashtable();
					if(relationNode.getNodeType() == Element.ELEMENT_NODE){
						//Putting attributes in Hashtable
						Element relationElement = (Element)relationNode;
						NodeList originList = relationElement.getElementsByTagName("origin");
						Element firstNameOrigin = (Element)originList.item(0);
						NodeList textList = firstNameOrigin.getChildNodes();
						table.put("origin", ((Node)textList.item(0)).getNodeValue().trim());
//						originList = relationElement.getElementsByTagName("connected");
//						firstNameOrigin = (Element)originList.item(0);
//						textList = firstNameOrigin.getChildNodes();
//						table.put("connected", ((Node)textList.item(0)).getNodeValue().trim());
						originList = relationElement.getElementsByTagName("x");
						firstNameOrigin = (Element)originList.item(0);
						textList = firstNameOrigin.getChildNodes();
						table.put("x", ((Node)textList.item(0)).getNodeValue().trim());
						originList = relationElement.getElementsByTagName("y");
						firstNameOrigin = (Element)originList.item(0);
						textList = firstNameOrigin.getChildNodes();
						table.put("y", ((Node)textList.item(0)).getNodeValue().trim());
						relations.add(table);
					}
				}
				compositionList.add(relations);
			}
		}
		return compositionList;
	}
	
	/**
	 * Saves all services and compositions in same manner that is presented in get methods.
	 * @param services ArrayList-Hashtable
	 * @param compositions ArrayList-Hashtable
	 */
	public void save(ArrayList services, ArrayList compositions){
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setValidating(false);
		try {
			//Creating root node and services/compositions structure
			doc = factory.newDocumentBuilder().newDocument();
			Element root = (Element) doc.createElement("bundles"); 
			doc.appendChild(root);
			Element servicesElement = (Element) doc.createElement("services");
			root.appendChild(servicesElement);
			Element compositionsElement = (Element) doc.createElement("compositions");
			root.appendChild(compositionsElement);
			
			Element bundleElement;
			Element idElement;
			for (int i = 0; i < services.size(); i++) {
				bundleElement = (Element)doc.createElement("bundle");
				idElement = (Element)doc.createElement("id");
				idElement.appendChild(doc.createTextNode((String) ((Hashtable)services.get(i)).get("id")));
				bundleElement.appendChild(idElement);
				servicesElement.appendChild(bundleElement);
			}
			
			Element compositionElement;
			Element relationElement;
			//Traversing compositions
			for (int i = 0; i < compositions.size(); i++) {
				//Creating compositionelement
				System.out.println("Composition: "+i);
				compositionElement = (Element) doc.createElement("composition");
				ArrayList relations = (ArrayList) compositions.get(i);
				Element originElement;
//				Element connectedElement;
				Element xElement;
				Element yElement;
				//Traversing relations of composition "i"
				for (int j = 0; j < relations.size(); j++) {
					//Creating relationelement
					System.out.println("Relation: "+j);
					relationElement = (Element) doc.createElement("relation");
					//Adding relation data
					System.out.println("her");
					originElement = (Element) doc.createElement("origin");
					System.out.println("her2");
					originElement.appendChild(doc.createTextNode((String) ((Hashtable)relations.get(j)).get("origin")));
//					connectedElement = (Element) doc.createElement("connected");
//					connectedElement.appendChild(doc.createTextNode((String) ((Hashtable)relations.get(j)).get("connected")));
					xElement = (Element) doc.createElement("x");
					xElement.appendChild(doc.createTextNode((String) ((Hashtable)relations.get(j)).get("x")));
					yElement = (Element) doc.createElement("y");
					yElement.appendChild(doc.createTextNode((String) ((Hashtable)relations.get(j)).get("y")));
					//Appending data to relation
					relationElement.appendChild(originElement);
//					relationElement.appendChild(connectedElement);
					relationElement.appendChild(xElement);
					relationElement.appendChild(yElement);
					//Appending relation to composition
					compositionElement.appendChild(relationElement);
				}
				//Appending composition to compositions
				compositionsElement.appendChild(compositionElement);
			}
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		write();
	}
	
	public void write(){
		try {
			// Prepare the DOM document for writing
			Source source = new DOMSource(doc);
			
			// Prepare the output file
			File file = new File(filename);
			if(!file.getParentFile().exists()){
				file.getParentFile().mkdirs();
			}
			Result result = new StreamResult(file);
			
			// Write the DOM document to the file
			Transformer xformer = TransformerFactory.newInstance().newTransformer();
			xformer.transform(source, result);
		} catch (TransformerConfigurationException e) {
		} catch (TransformerException e) {
		}
	}
	
	public static void main(String[] args){
		XMLManager man = new XMLManager("test.xml", false);
		List servicelist = man.getServiceList();
		for (int i = 0; i < servicelist.size(); i++) {
			System.out.println("Service "+i+"-----");
			System.out.println("   id: "+((Hashtable)servicelist.get(i)).get("id"));
		}
		System.out.println();
		List comps = man.getCompositionList();
		for (int i = 0; i < comps.size(); i++) {
			System.out.println("Composition "+i+"-----");
			ArrayList relations = (ArrayList) comps.get(i);
			for (int j = 0; j < relations.size(); j++) {
				System.out.println("   Relation-----");
				System.out.println("      origin: "+((Hashtable)relations.get(j)).get("origin"));
//				System.out.println("      connected: "+((Hashtable)relations.get(j)).get("connected"));
				System.out.println("      x: "+((Hashtable)relations.get(j)).get("x"));
				System.out.println("      y: "+((Hashtable)relations.get(j)).get("y"));
			}
		}
		System.out.println();
//		man.save(servicelist, comps);
//		man.write();
	}
}
