package weatherProxy.impl;

import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;

import weatherProxy.ServiceInterface;

public class Service implements ServiceInterface {

	private String copeurl = "http://localhost:8080/axis/services/copeserver";

	// this computer has a static IP, and is available over the internet
//	private String serviceurl = "http://129.241.219.153:8080/axis/services/weatherservice";
	private String serviceurl = "http://localhost:8080/axis/services/weatherservice";
	
	private String name = "Weather Service";

	protected File icon;

	private long id;

	private JPanel holder;

	protected JFrame main;

	private String copeIP;

	private String[] curHeaders;

	private String[] curValues;

	private Properties props;

	private boolean subscribing = false;

	public Service() {
		System.out.println(serviceurl);
	}

	public void event(String description, Object oldValue, Object newValue) {
		System.out.println(getName() + " received command: event");
	}

	public void openInterface() {
		System.out.println(getName() + " received command: openInterface");
		main = new JFrame("Weather Service (God mode)");

		main.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.out.println("windowClosing");
				if(subscribing)
					unsubscribe();
			}

//			public void windowOpened(WindowEvent e) {
//				System.out.println("windowOpened");
//				subscribe();
//			}
		});

		holder = new JPanel();
		holder.setLayout(new GridLayout(0,3));
		
		JButton sub = new JButton("Subscribe");
		sub.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				subscribe();
			}
		});
		JButton unsub = new JButton("Unsubscribe");
		unsub.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				unsubscribe();
			}
		});
		JButton getWeather = new JButton("Get Current Weather Data");
		getWeather.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Object[] obj = getWeather();
				weatherUpdate((String[]) obj[0], (String[]) obj[1]);
			}
		});

		holder.add(sub);
		holder.add(unsub);
		holder.add(getWeather);

		main.setSize(400, 60);
		main.setContentPane(holder);
		main.setVisible(true);
	}


	public Properties getDescription() {
		System.out.println(getName() + " received command: getDescription");
		props = new Properties();
		props.put("connection.originating.1", "weatherConnection");
		props.put("connection.originating.2", "alarmConnection");
		
		return props;
	}

	public void connect(String description, String source, String target) {
		System.out.println(getName() + " received command: connect");
		Communicator.sendRequest(serviceurl, "connect", new Object[] { description, source, target });	
	}

	public void disconnect(String description, String source, String target) {
		System.out.println(getName() + " received command: disconnect");
		Communicator.sendRequest(serviceurl, "disconnect", new Object[] { description, source, target });
	}

	public void setID(long id) {
		System.out.println(getName() + " received command: setID");
		this.id = id;
	}

	public void ping() {
		System.out.print(getName() + " received ping");
		Communicator.sendRequest(copeurl, "pong", new Object[] { new Long(id) });
	}

	public void reRegisterWithCope() {
		System.out.println(getName() + " received command: reRegisterWithCope");
		Communicator.sendRequest(copeurl, "reRegisterService", new Object[] { new Long(id) });
	}

	public String getName() {
		return name;
	}

	public void setURL(String url) {
		serviceurl = url;
	}

	public String getURL() {
		return serviceurl;
	}
	
	public String getIconPath() {
		try {
			if (icon.exists())
				icon.delete();

			icon.createNewFile();

			FileInputStream fis = (FileInputStream) getClass().getResourceAsStream("/icon.png");
			FileOutputStream fos = new FileOutputStream(icon);
			int b;
			while ((b = fis.read()) != -1) {
				fos.write(b);
			}

			fos.flush();
			fos.close();

			fis.close();

			if (icon != null)
				return icon.getPath();

		} catch (IOException e) {
			e.printStackTrace();
		}

		return null;
	}

	/************************************
	 * 
	 * Custom stuff
	 *
	 ************************************/
	public void subscribe() {
		// getting ip from cope server
		copeIP = (String) Communicator.sendRequest(copeurl, "getIp", new Object[]{});
		// subscribing with correct IP
//		Communicator.sendRequest(serviceurl, "subscribe", new Object[] { "http://" + copeIP + ":8080/axis/services/alarmproxy" });
		Communicator.sendRequest(serviceurl, "subscribe", new Object[] { "http://localhost:8080/axis/services/alarmproxy" });
		subscribing = true;
	}

	private void unsubscribe() {
//		Communicator.sendRequest(serviceurl, "unsubscribe", new Object[] { "http://" + copeIP + ":8080/axis/services/alarmproxy" });
		Communicator.sendRequest(serviceurl, "unsubscribe", new Object[] { "http://localhost:8080/axis/services/alarmproxy" });
		subscribing = false;
	}


	private Object[] getWeather() {
		return (Object[]) Communicator.sendRequest(serviceurl, "getWeather", new Object[] { });
	}
	
	public void weatherUpdate(String[] headers, String[] values) {
		holder.removeAll();
		holder.setLayout(new GridLayout(0, 4));

		curHeaders = headers;
		if(values != null)
			curValues = values;
		else
			curValues = new String[curHeaders.length];
		
		for (int i = 0; i < curHeaders.length; i++) {

			JLabel label = new JLabel(curHeaders[i]);
			holder.add(label);

			try {
				JSpinner spinner = new JSpinner();
				spinner.setValue(new Double(Double.parseDouble(curValues[i])));

				holder.add(spinner);
			} catch (Exception e) {
				// couldn't parse to double, making it text
				if(curValues != null && curValues[i] != null) {
					JTextField text = new JTextField(curValues[i]);
					holder.add(text);
				} else {
					JTextField text = new JTextField((int) (Math.random()*127));
					holder.add(text);
				} 
				 
//				text.setEditable(false);
			}

		}
		


		JButton sub = new JButton("Subscribe");
		sub.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				subscribe();
			}
		});
		JButton unsub = new JButton("Unsubscribe");
		unsub.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				unsubscribe();
			}
		});
		JButton getWeather = new JButton("Get Current Weather Data");
		getWeather.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Object[] obj = getWeather();
				weatherUpdate((String[]) obj[0], (String[]) obj[1]);
			}
		});
		JButton button = new JButton("Update weather");
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateWeather();
			}
		});

		holder.add(sub);
		holder.add(unsub);
		holder.add(getWeather);
		holder.add(button);
		
		main.pack();

		main.repaint();

		holder.updateUI();
	}

	protected void updateWeather() {
		System.out.println("Updating weather (in god mode)");

		if(curValues==null)
			curValues = new String[curHeaders.length];
		
		Component[] components = holder.getComponents();
		
		int k = 0;
		for (int j = 0; j < components.length; j++) {
			if (components[j] != null) {
				if (components[j] instanceof JSpinner) {
					String text = ((JSpinner) components[j]).getValue().toString();
					curValues[k] = text.length()!=0?text:"0";
					k++;
				} else if (components[j] instanceof JTextField) {
					String text = ((JTextField) components[j]).getText();
					curValues[k] = text.length()!=0?text:"0";
					k++;
				}
			}
		}

		Communicator.sendRequest(serviceurl, "updateWeather", new Object[]{curHeaders, curValues});

	}

}
