package weatherService.impl;

import java.util.ArrayList;
import java.util.List;

import sun.security.krb5.internal.crypto.dk.DkCrypto;

import weatherService.ServiceInterface;

public class Service implements ServiceInterface {

	List weatherDestination = new ArrayList();
	
	List alarmDestination = new ArrayList();

	List subscribers = new ArrayList();
	
	protected boolean dbConnOK = false;
	
	String[] curHeaders = new String[]{
			"station_id", 
			"date", 
			"time", 
			"average_windspeed", 
			"wind_direction", 
			"gust_windspeed", 
			"temperature", 
			"outdoor_humidity", 
			"barometer", 
			"daily_rainfall", 
			"monthly_rainfall", 
			"yearly_rainfall", 
			"rain_rate", 
			"max_rain_rate_curent_day", 
			"indoor_temperature", 
			"indoor_humidity", 
			"soil_temperature", 
			"yesterday_rainfall", 
			"current_windchill",  
			"current_humidex", 
			"max_daily_temperature",  
			"min_daily_temperature", 
			"icon_type",  
			"current_weather_desc", 
			"barometer_trend_last_hour",  
			"max_gust_current_day", 
			"dew_point_temperature",  
			"cloud_height",  
			"max_humidex", 
			"min_humidex",  
			"max_windchill",  
			"min_windchill", 
			"davis_vp_uv",  
			"max_heat_index",  
			"min_heat_index", 
			"heat_index"			
	};

	String[] curValues = new String[curHeaders.length];

	String serviceurl = "";
	
	public void connect(String description, String source, String target) {
		System.out.println("Got connection attempt: " + description + ", " + source + ", " + target);
		if (description.equalsIgnoreCase("weatherConnection")) {
			if(!weatherDestination.contains(target)) {
				weatherDestination.add(target);
				System.out.println("WEATHERSERVICE: Added new datadestination: " + target);
			}
		} else if (description.equalsIgnoreCase("alarmConnection")) {
			// setting my own address
			serviceurl = source;
			if(!alarmDestination.contains(target)) {
				alarmDestination.add(target);
				System.out.println("WEATHERSERVICE: Added new alarmdestination: " + target);
			}
		}
	}

	public void disconnect(String description, String source, String target) {
		if (description.equalsIgnoreCase("weatherConnection")) {
			if(weatherDestination.contains(target)) {
				weatherDestination.remove(target);
			}
		} else if (description.equalsIgnoreCase("alarmConnection")) {
			if(alarmDestination.contains(target)) {
				alarmDestination.remove(target);
			}
		}
	}

	public void updateWeather(final String[] headers, final String[] values) {
//		curHeaders = headers;
		curValues = values;

		checkForAlarms(headers, values);
		
		// update subscribers i.e. proxies
		for (int i = 0; i < subscribers.size(); i++) {
			final String addr = (String) subscribers.get(i);
			new Thread(new Runnable() {
				public void run() {
					Communicator.sendRequest(addr, "weatherUpdate",
							new Object[] { curHeaders, curValues });
				}
			}).start();
		}

		// send weather update to those that are connected
		for (int i = 0; i < weatherDestination.size(); i++) {
			final String addr = (String) weatherDestination.get(i);
			new Thread(new Runnable() {
				public void run() {
					System.out.println("Weather Serivce: Sending data to " + addr);
					Communicator.sendRequest(addr, "setWeatherData", new Object[] { headers, values });
				}
			}).start();
		}
	}

	
	private void checkForAlarms(String[] headers, String[] values) {
		for (int i = 0; i < headers.length; i++) {
			if(headers[i] != null && headers[i].equalsIgnoreCase("temperature"))
				if(Double.parseDouble(values[i])>=21)
					sendAlarm("WARNING: Temperature is over 21 degrees. Current value is " + values[i] + ".");
		}
	}

	private void sendAlarm(String alarm) {
		if(serviceurl.length()==0)
			return;
		for (int i = 0; i < alarmDestination.size(); i++) {
			System.out.println("Sending alarm to " + serviceurl);
			Communicator.sendRequest((String) alarmDestination.get(i), "sendAlarm", new Object[]{alarm, serviceurl});
		}
	}

	public void subscribe(String address) {
		System.out.println("Subscribe request from " + address);
		if (!subscribers.contains(address))
			subscribers.add(address);
		printSubscribers();
	}

	public void unsubscribe(String address) {
		System.out.println("Unubscribe request from " + address);
		if (subscribers.contains(address))
			subscribers.remove(address);
		printSubscribers();
	}

	private void printSubscribers() {
		System.out.println("Current subscribers: ");
		for (int i = 0; i < subscribers.size(); i++) {
			System.out.println(subscribers.get(i));
		}
	}

	public Object[] getWeather() {
		if(!dbConnOK) {
			for (int i = 0; i < curHeaders.length; i++) {
				curValues[i] = "" + (int) (Math.random()*127);
			}
		}
		return new Object[] { curHeaders, curValues };	
	}
}
