package weatherService.impl;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Dictionary;
import java.util.Hashtable;

import org.osgi.framework.BundleContext;
import org.osgi.framework.Constants;
import org.osgi.framework.ServiceRegistration;
import org.osgi.service.cm.ConfigurationException;
import org.osgi.service.cm.ManagedService;

import weatherService.ServiceInterface;

public class ServiceManaged extends Service implements ManagedService, Runnable {

	private Hashtable props;

	private ServiceRegistration registration;

	String[] headers;

	String[] values;

	private Statement stmt = null;

	private ResultSet rs = null;

	private Connection conn;

	private boolean running = false;

	private boolean querying = false;

	private String columns = "station_id, "
			+ "date, "
			+ "time, "
			+ "average_windspeed, "
			+ "wind_direction, "
			+ "gust_windspeed, "
			+ "temperature, "
			+ "outdoor_humidity, "
			+ "barometer, "
			+ "daily_rainfall, "
			+ "monthly_rainfall, "
			+ "yearly_rainfall, "
			+ "rain_rate, "
			+ "max_rain_rate_curent_day, "
			+ "indoor_temperature, "
			+ "indoor_humidity, "
			+ "soil_temperature, "
			// + "forecast_icon, "
			// + "wmr968_extra_temperature, "
			// + "wmr968_extra_humidity, "
			// + "wmr968_extra_sensor_number, "
			+ "yesterday_rainfall, "
			+
			// "extra_temperature_sensor_2, " +
			// "extra_temperature_sensor_3, " +
			// "extra_temperature_sensor_4, " +
			// "extra_temperature_sensor_5, " +
			// "extra_temperature_sensor_6, " +
			// "extra_temperature_sensor_7, " +
			// "extra_humidity_2, " +
			// "extra_humidity_3, " +
			// "extra_humidity_4, " +
			// "hour, " +
			// "minute, " +
			// "second, " +
			// "station_name, " +
			// "dallas_1_wire_lightning_count, " +
			// "actual_solar_reading, " +
			// "day, " +
			// "month, " +
			// "wmr968_battery_level_1, " +
			// "wmr968_battery_level_2, " +
			// "wmr968_battery_level_3, " +
			// "wmr968_battery_level_4, " +
			// "wmr968_battery_level_5, " +
			// "wmr968_battery_level_6, " +
			// "wmr968_battery_level_7, " +
			"current_windchill, " 
			+ "current_humidex, "
			+ "max_daily_temperature, " 
			+ "min_daily_temperature, "
			+ "icon_type, " 
			+ "current_weather_desc, "
			+ "barometer_trend_last_hour, " 
			+ "max_gust_current_day, "
			+ "dew_point_temperature, " 
			+ "cloud_height, " 
			+ "max_humidex, "
			+ "min_humidex, " 
			+ "max_windchill, " 
			+ "min_windchill, "
			+ "davis_vp_uv, " 
			+ "max_heat_index, " 
			+ "min_heat_index, "
			+ "heat_index";

	public ServiceManaged(BundleContext context) {
		props = new Hashtable();
		props.put(Constants.SERVICE_PID, "weatherService"
				+ this.getClass().getName());
		props.put("SOAP.service.name", "weatherservice");

		try {
			Class.forName("com.mysql.jdbc.Driver");
			conn = DriverManager.getConnection("jdbc:mysql://localhost/vp2?" + "user=monty&password=monty");
			dbConnOK = true;
			running = true;
		} catch (SQLException e) {
			// handle any errors
			System.out.println("SQLException: " + e.getMessage());
			System.out.println("SQLState: " + e.getSQLState());
			System.out.println("VendorError: " + e.getErrorCode());
		} catch (ClassNotFoundException e) {
			System.out.println("ClassNotFoundException: " + e.getMessage());
		}
		// start data collection
		new Thread(this).start();
	}

	public synchronized void open(BundleContext context) {
		System.out.print("Registering with framework... ");
		registration = context.registerService(
				"weatherService.ServiceInterface", ((ServiceInterface) this),
				props);

	}

	public void updated(Dictionary arg0) throws ConfigurationException {

	}

	/**
	 * Used for unloading the CoPE bundle Stores session
	 */

	public void unload() {
		registration.unregister();

		running = false;

		// wait for a potensial query to finish
		while (querying) {
			try {
				Thread.sleep(500);
			} catch (InterruptedException ignored) {
			}
		}
		if (rs != null) {
			try {
				rs.close();
			} catch (SQLException sqlEx) { // ignore }
				rs = null;
			}
		}
		if (stmt != null) {
			try {
				stmt.close();
			} catch (SQLException sqlEx) { // ignore }
				stmt = null;
			}
		}

		dbConnOK = false;
	}

	public void getLatestWeatherData() {
		if (dbConnOK) {
			querying = true;

			try {
				stmt = conn.createStatement();

				rs = stmt
						.executeQuery("SELECT "
								+ columns
								+ " FROM wx_data ORDER BY date DESC, time DESC LIMIT 1");

				ResultSetMetaData metaData = rs.getMetaData();
				headers = new String[metaData.getColumnCount()];
				values = new String[headers.length];

				// to make sure we get all rows
				rs.beforeFirst();
				boolean queryReturnedValues = rs.next();

				for (int i = 0; i < headers.length - 1; i++) {
					headers[i] = metaData.getColumnName(i + 1);

					if (queryReturnedValues)
						values[i] = rs.getObject(i + 1).toString();
					else
						values[i] = "";
				}

				rs.close();
				stmt.close();

			} catch (SQLException e) {
				System.out.println("SQLException: " + e.getMessage());
				System.out.println("SQLState: " + e.getSQLState());
				System.out.println("VendorError: " + e.getErrorCode());
			}

			querying = false;
		} else {
			System.out.println("checking weather failed");
		}
	}

	public void run() {
		while (running) {
			getLatestWeatherData();
			updateWeather(headers, values);
			try {
				Thread.sleep(5000); // get new weather data every 5 secs
			} catch (InterruptedException e) {
				// ignored
			}
		}
	}
}
